      Program pwr2lyr

* Generate stair step model from power law

* Dates: 08/17/99 - Written for specific application (extrapolating 
*                   the velocities at Hollister Airport)
*        09/09/99 - Generalized, checking for z1 = 0.0, and including the
*                   option to choose layer thickness such that they have 
*                   equal travel times (C. Mueller's suggestion).
*        05/12/00 - Option to specify nlayers or dt for constant
*                   time layers; work on output
*        10/09/00 - Added check for pathological cases pwr >= 1, z1 = 0.0
*        10/10/00 - Treat special case when pwr = 1, z1 > 0.0
!        04/11/11 - Add cumulative travel time from the surface and average velocity 
!                   to the output.
!                 - Add zoffset (to be able to generate model corresponding to layers stripped off of
!                   the top).
!        04/12/11 - Add iterations to choose zoffset to yield a desired Vs30
!                 - Print out input parameters in output file

      real z(0:1000), vpwr(0:1000), ttpwr(0:1000), v0
      real vlyr(1000), zstep(0:2000), vstep(0:2000),
     :                ttlyr(0:2000)
      character answr*1, f_stem*80, f_mdl*80, f_vel*80, buf*10
      logical vel_units_mps, specify_time_across_layer
      
      z = 0.0
      vpwr = 0.0
      ttpwr = 0.0
      vlyr = 0.0
      zstep = 0.0
      vstep = 0.0
      ttlyr = 0.0

      write(*, '(a)') 
     :  ' Enter z0, v0, pwr (.ne. 1.0): '
      read(*,*) z0, v0, pwr
      
      write(*, '(a)') ' Are the velocity units m/s? (Y/N)'
      buf = ' '
      read(*,'(a)') buf
      call trim_c(buf, nc_buf)
      call upstr(buf(1:1))
      if (buf(1:1) == 'Y') then
        vel_units_mps = .true.
      else
        vel_units_mps = .false.
      end if      
      
      write(*, '(a)') ' Enter desired Vs30 (0 = skip iterations):'
      write(*, '(a)') 
     :       '   (make units are consistent with power law model)'
      read(*,*) Vs30
      
      if (vs30 == 0.0) then
        write(*,'(a)') ' Enter zoffset: '
        read(*,*) zoffset
          z1 = 0.0
          if (vel_units_mps) then
            z2 = 30.0
          else
            z2 = 0.030
          end if
          tt_0_30 = tt(z1+zoffset, z2+zoffset, v0, z0, pwr)
          vs30_calc = z2/tt_0_30
          write(*,'(a,1x,es10.3,1x,a,1x,es10.3)') 
     :         ' zoffset = ', zoffset, ' gives Vs30 = ', vs30_calc
      else      
        DO
          write(*,'(a)') ' Enter a new zoffset (<0 to quit): '
          read(*,*) zoffset_new
          if(zoffset_new < 0.0) then
            EXIT
          end if
          zoffset = zoffset_new
          z1 = 0.0
          if (vel_units_mps) then
            z2 = 30.0
          else
            z2 = 0.030
          end if
          tt_0_30 = tt(z1+zoffset, z2+zoffset, v0, z0, pwr)
          vs30_calc = z2/tt_0_30
          write(*,'(a,1x,es10.3,1x,a,1x,es10.3)') 
     :         ' zoffset = ', zoffset, ' gives Vs30 = ', vs30_calc
        END DO
      endif
      
      z2_for_vs30 = z2
        

      write(*, '(a)') 
     :  ' Type of spacing: 0 = equal times; 1 = log depth: '
      read(*,*) i_type_space

* Generate layering:

      IF (i_type_space == 0) THEN   ! equal time layers:
        write(*, '(a)') ' Specify time across layers (y/n)?'
        answr = ' '
        read(*,'(a)') answr
        call upstr(answr)
        
        IF (answr == 'Y') THEN
          specify_time_across_layer = .true.
        ELSE
          specify_time_across_layer = .false.
        END IF
        
        IF (specify_time_across_layer) THEN
        
          DO
            write(*, '(a)') ' Enter z1, z2, dt: '  ! specify time across layer
            read(*,*) z1, z2, tt_lyr
            IF( pwr >= 1.0 .and. z1 == 0.0) THEN
              write(*,'(a)') ' pwr >= 1.0, z1 is pathological; '//
     :                  'enter a new value for z1!!! (ctl-c to quit)'
              write(*,*)
              CYCLE
            ELSE
              EXIT
            END IF
          END DO  
            
          tt_z1_z2 = tt(z1+zoffset, z2+zoffset, v0, z0, pwr) 
          nlayers = tt_z1_z2/tt_lyr +0.1
          
        ELSE
        
          DO
            write(*, '(a)') ' Enter z1, z2, nlayers: ' ! spcfy # equal time layers
            read(*,*) z1, z2, nlayers
            IF( pwr >= 1.0 .and. z1 == 0.0) THEN
              write(*,'(a)') ' pwr >= 1.0, z1 is pathological; '//
     :                  'enter a new value for z1!!! (ctl-c to quit)'
              write(*,*)
              CYCLE
            ELSE
              EXIT
            END IF  
          END DO
          
          tt_lyr = 
     :         tt(z1+zoffset, z2+zoffset, v0, z0, pwr)/real(nlayers)
     
        END IF

        one_pwr = 1.0 - pwr
        tfctr1 = exp((v0/z0)*tt_lyr)
        tfctr2 = v0*one_pwr*tt_lyr/z0**pwr
        z(0) = z1
        DO i = 1, nlayers
          IF (pwr == 1.0) THEN
            z(i) = (z(i-1)+zoffset) * tfctr1 - zoffset
          ELSE
            z(i) = ((z(i-1)+zoffset)**one_pwr + tfctr2)**(1.0/one_pwr)
     :               - zoffset
          END IF
        END DO
        
!        IF (z(nlayers) < z2) THEN
!          nlayers = nlayers + 1
!          z(nlayers) = z2
!        END IF

      ELSE                            ! log spacing:

        write(*, '(a)') ' Enter z1 (.ne. 0.0), z2, nlayers: '
        read(*,*) z1, z2, nlayers
        dlz = (alog10(z2/z1))/real(nlayers)
* NEED TO INSERT AN INITIAL CONSTANT VELOCITY LAYER (LATER!!!)
        z(0) = z1
        DO i = 1, nlayers
          z(i) = z1*10.0**(real(i)*dlz)
        END DO
        
      END IF
      
* Note: if want depths rounded to whole numbers, use anint intrinsic function

* Generate times:
      
      vpwr(0) = vel(z(0)+zoffset, v0, z0, pwr)
      ttpwr(0) = 0.0
      do i = 1, nlayers
        vpwr(i) = vel(z(i)+zoffset, v0, z0, pwr)
        ttpwr(i) = tt(z(0)+zoffset, z(i)+zoffset, v0, z0, pwr)
      end do

* Generate constant velocity layers:

      do i = 1, nlayers
        vlyr(i) = (z(i) - z(i-1))/(ttpwr(i) - ttpwr(i-1))
      end do


* OUTPUT:

* Get stem name of output files:
      f_stem = ' '
      write(*,'(a)') ' Enter stem name of output files:'
      read(*,'(a)') f_stem
      call trim_c(f_stem, nchar_stem)

* write model for use in site_amp (and from there into nrattle, via
* f4nrattl):

      f_mdl = ' '
      f_mdl = f_stem(1:nchar_stem)//'.mdl'

      write(*,'(2a)') ' Writing model file = ', f_mdl

      call get_lun(nu_mdl)
      open(unit=nu_mdl,file=f_mdl,status='unknown')
      
      if (z1 == 0.0 .and. z2 >= z2_for_vs30) then
        write(nu_mdl,'(a)') 
     :           ' z0, v0, pwr, zoffset, z1, z2, vs30_calc = '
        write(nu_mdl,'(8(1x,es11.4))') 
     :           z0, v0, pwr, zoffset, z1, z2, vs30_calc
      else
        write(nu_mdl,'(a)') ' z0, v0, pwr, zoffset, z1, z2 = '
        write(nu_mdl,'(8(1x,es11.4))') 
     :           z0, v0, pwr, zoffset, z1, z2
      end if
      
      IF (i_type_space == 0) THEN   ! equal time layers:
        IF (specify_time_across_layer) THEN
          write(nu_mdl,'(a,1x, es10.3)') 
     :      '  equal time layers with specified time across layer = ', 
     :                 tt_lyr
        ELSE
          write(nu_mdl,'(a, 1x, i4)') 
     :   '  equal time layers with time across layers from nlayers = ', 
     :                 nlayers 
        END IF
      ELSE
          write(nu_mdl,'(a)') 
     :      '  nlayers logarithmically spaced layers from z1 to z2' 
      END IF
      
      write(nu_mdl,'(t3,a, t20,a, t30,a, t42,a)')
     :  'thickness', 'vel', 'dens', '1/Q'
      do i = 1, nlayers
        write(nu_mdl,'(4(1x,es10.3))')
     :    z(i) - z(i-1), vlyr(i), 0.0, 0.0
      end do
      close(nu_mdl)
        
* Write velocity in form for import into CoPlot
* Note that to get separate columns for input data, shear
* velocities, and model travel times requires
* doing the output in three steps.

      idx = -1
      do i = 1, nlayers
        idx = idx+1
        zstep(idx) = z(i-1)
        vstep(idx) = vlyr(i)

        idx = idx+1
        zstep(idx) = z(i)
        vstep(idx) = vlyr(i)
      end do
      nstep = idx
      
      cumtt = 0.0
      do i = 1, nstep
        cumtt = cumtt + (zstep(i)-zstep(i-1))/vstep(i)
        ttlyr(i) = cumtt
      end do


* write model:
      f_vel = ' '
      f_vel = f_stem(1:nchar_stem)//'.col'
      write(*,'(2a)') ' Writing stairstep velocity file = ', f_vel

      call get_lun(nu_col)
      open(unit=nu_col,file=f_vel,status='unknown')
      
      if (z1 == 0.0 .and. z2 >= z2_for_vs30) then
        write(nu_col,'(a)') 
     :           ' z0, v0, pwr, zoffset, z1, z2, vs30_calc = '
        write(nu_col,'(8(1x,es11.4))') 
     :           z0, v0, pwr, zoffset, z1, z2, vs30_calc
      else
        write(nu_col,'(a)') ' z0, v0, pwr, zoffset, z1, z2 = '
        write(nu_col,'(8(1x,es11.4))') 
     :           z0, v0, pwr, zoffset, z1, z2
      end if
      
      IF (i_type_space == 0) THEN   ! equal time layers:
        IF (specify_time_across_layer) THEN
          write(nu_col,'(a,1x, es10.3)') 
     :      '  equal time layers with specified time across layer = ', 
     :                 tt_lyr
        ELSE
          write(nu_col,'(a, 1x, i4)') 
     :   '  equal time layers with time across layers from nlayers = ', 
     :                 nlayers 
        END IF
      ELSE
          write(nu_col,'(a)') 
     :      '  nlayers logarithmically spaced layers from z1 to z2' 
      END IF
      
      write(nu_col,'( 
     :                 4x,a, 7x,a, 2x,a, 
     :                 7x,a, 6x,a, 4x,a,
     :                 7x,a, 7x,a, 6x,a, 4x,a
     :                                            )')
     :  'zoffset', 'zpwr', 'z+zoffset', 'vpwr', 'ttpwr', 'vavgpwr', 
     :  'zlyr', 'vlyr', 'ttlyr', 'vavglyr'
      write(nu_col,'(5(1x,es10.3), 11x, 5(1x,es10.3))')
     :    zoffset, z(0), z(0)+zoffset, 
     :    vpwr(0), ttpwr(0),  
     :    zstep(0), vstep(0), ttlyr(0)  
      do i = 1, nlayers
        write(nu_col,'(10(1x,es10.3))')
     :    zoffset, z(i), z(i)+zoffset, 
     :    vpwr(i), ttpwr(i), z(i)/ttpwr(i),
     :    zstep(i), vstep(i), ttlyr(i), zstep(i)/ttlyr(i)
      end do
      do i = nlayers+1, nstep
        write(nu_col,'(66x,4(1x,es10.3))')
     :    zstep(i), vstep(i), ttlyr(i), zstep(i)/ttlyr(i)
      end do

      close(nu_col)  

      end

* -------------------------------------------------------------------- tt
      function tt(z1, z2, v0, z0, pwr)
* travel time between depths z1 and z2 for the velocity function
*   v = v0*(z/z0)**pwr, where z1 < z2 and pwr /= 1.0.

* Dates: 08/16/99 - written by D. Boore
*        09/09/99 - added check for z1 = 0.0
*        10/09/00 - added check for pwr = 1.0
*        10/10/00 - deal with special case when pwr = 1.0

      if (z1 == 0.0) then
        if (z2 == 0.0) then
          tt = 0.0
        else
          if (pwr >= 1.0) then
            tt = -999
          else
            tt = (z2*(z0/z2)**pwr)/((1.0 - pwr)*v0)
          end if
        end if
      else
        if (pwr == 1.0) then
          tt = (z0/v0) * alog(z2/z1)
        else
          tt = (z2*(z0/z2)**pwr - z1*(z0/z1)**pwr)/((1.0 - pwr)*v0)
        end if
      end if

      return
      end
* -------------------------------------------------------------------- tt

* -------------------------------------------------------------------- vel
      function vel(z, v0, z0, pwr)
* velocity for power law function

* Dates: 08/16/99 - written by D. Boore

      vel = v0*(z/z0)**pwr

      return
      end
* -------------------------------------------------------------------- vel

      include 'site_amp_util_subs.for'

 
     
