
         Program Impulse_Response_From_Nrattle_Output


! This program computes the time domain impulse response corresponding to a Nrattle
! output.  Note that it is crucial that the nfreq from Nrattle
! is compatible with the FFT: npts = 2*(nfreq - 1) must be a power of 2.
 
! If nfreq and fhigh are being specified for use in creating a time series via an FFT, 
!         nfreq = npts/2 + 1
!         fhigh = 1.0/(2.0*dt) (the Nyquist frequency)
! so
!         npts = 2*(nfreq - 1)
!         sps = 2.0*fhigh
!         dt = 1.0/sps

!
!!Control file for program Impulse_Response_From_Nrattle_Output
!! As many comment lines as desired, each starting with "!"
!! The string "pp:" indicates a new set of processing parameters
!! to be applied to the following smc files.  The parameters are given on the
!! following lines.
!! Revision of program involving a change in the control file on this date:
!   12/20/15
!!Name of summary file:
! impulse_response_from_nrattle_output.sum
!!Name of complex transfer function, made with nrattle.
! composite_model.mdl.aoi_00.for_ts.q50.nrattle_complex_spectra.out
!!Name of output file
! composite_model.mdl.aoi_00.for_ts.q50.nrattle_complex_spectra.ts.smc
!time shift for impulse response [in freq dom, multiply FS by exp(+i*omega*tshift)]:
! 1.0
!Stop

! Dates: 12/20/15 - Written by D. Boore, based on C:\svel\i10_sat\decon_ts.for

      real real_head(50),S(50000),  freq(50000),
     : areal_nrattle(50000),aimag_nrattle(50000)

      Complex CARG,CEXP,CTEMP,T(50000),FS(50000) 
      
      character comments(500)*80, 
     : f_sum*200, f_fs*200, f_out*200, buf*500,
     : smc_precision*20
     
      character  f_ctl*200, ctl_cmmnts(60)*100

      character date_ctl_correct*8, date_ctl_in*30
      logical  file_exist 
      
      character char_head(11)*80
      integer int_head(48), NPW2 

      file_exist = .false.
      
      twopi = 8.0 * atan(1.0)

      DO
        f_ctl = ' '
        write(*, '(a)') 
     :    ' Enter name of control file '//
     :    '("Enter" = impulse_response_from_nrattle_output.ctl;'//
     :    ' ctl-brk to quit): '
        read(*, '(a)') f_ctl
        call trim_c(f_ctl, nc_f_ctl)
        if (f_ctl(1:4) .eq. '    ') 
     :        f_ctl = 'impulse_response_from_nrattle_output.ctl'
        call trim_c(f_ctl, nc_f_ctl)
        inquire(file=f_ctl(1:nc_f_ctl), exist=file_exist)
        IF (file_exist) EXIT
        write(*,'(a)') ' ******* FILE '//f_ctl(1:nc_f_ctl)//
     :                 ' DOES NOT EXIST, '//
     :                 'ENTER A VALID FILE NAME ******* '
      END DO
      call get_lun(nu_ctl)
      open(unit=nu_ctl,file=f_ctl(1:nc_f_ctl),
     :     status='unknown')

!Check version of control file
      date_ctl_correct = ' '
      date_ctl_correct = '12/20/15'
      call trim_c(date_ctl_correct,nc_date_ctl_correct)

      call skipcmnt(nu_ctl,ctl_cmmnts, nc_ctl_cmmnts)
      date_ctl_in = ' '
      read(nu_ctl,'(a)') date_ctl_in
      call trim_c(date_ctl_in,nc_date_ctl_in)
      
      if (date_ctl_correct(1:nc_date_ctl_correct) /=
     :    date_ctl_in(1:nc_date_ctl_in)) then
        write(*,'(a)') 
     :     ' The control file has the wrong date ('//
     :       date_ctl_in(1:nc_date_ctl_in)//'); '//
     :      'the proper date is '//date_ctl_correct
        write(*,'(a)') 
     :     ' STOP!'
        close(nu_ctl)
        stop
      end if
      
      call skipcmnt(nu_ctl,ctl_cmmnts, nc_ctl_cmmnts)
      f_sum = ' '
      read(nu_ctl, '(a)') f_sum
      call trim_c(f_sum, nc_f_sum)
      call get_lun(nu_sum)
      open(unit=nu_sum,file=f_sum(1:nc_f_sum),
     :     status='unknown')


!Get file name with complex output

      call skipcmnt(nu_ctl,ctl_cmmnts, nc_ctl_cmmnts)
      f_fs = ' '
      read(nu_ctl, '(a)') f_fs
      call trim_c(f_fs, nc_f_fs)
      call get_lun(nu_fs)
      open(unit=nu_fs,file=f_fs(1:nc_f_fs),
     :     status='unknown')


!Get file name for output

      call skipcmnt(nu_ctl,ctl_cmmnts, nc_ctl_cmmnts)
      f_out = ' '
      read(nu_ctl, '(a)') f_out
      call trim_c(f_out, nc_f_out)
      call get_lun(nu_out)
      open(unit=nu_out,file=f_out(1:nc_f_out),
     :     status='unknown')

!Get tshift

      call skipcmnt(nu_ctl,ctl_cmmnts, nc_ctl_cmmnts)
      read(nu_ctl, *) tshift
      write(nu_sum,'(1x,a, 1x,es11.4)') ' tshift = ', tshift
      write(nu_sum,'(1x,a, 1x,es11.4)') ' twopi = ', twopi

!Finished reading input:

      close(nu_ctl)
      
!Extract nfreq, fhigh, nlayers:

      call skip(nu_fs,5)
      buf = ' '
      read(nu_fs,'(a)') buf
      call trim_c(buf, nc_buf)
      read(buf(index(buf(1:nc_buf),'=')+1:nc_buf),*) nfreq
      write(nu_sum,'(1x,a, 1x,i6)') ' nfreq = ', nfreq
      
      buf = ' '
      read(nu_fs,'(a)') buf
      call trim_c(buf, nc_buf)
      read(buf(index(buf(1:nc_buf),'=')+1:nc_buf),*) fhigh
      write(nu_sum,'(1x,a, 1x,es11.4)') ' fhigh = ', fhigh
      
      call skip(nu_fs,4)
      buf = ' '
      read(nu_fs,'(a)') buf
      call trim_c(buf, nc_buf)
      read(buf(index(buf(1:nc_buf),'=')+1:nc_buf),*) nlayers
      write(nu_sum,'(1x,a, 1x,i6)') ' nlayers = ', nlayers
       
!Compute npw2 and make sure that it is a power of 2.
      npts = 2*(nfreq - 1)
      ln2_npts = int(alog(real(npts))/alog(2.0))
      write(nu_sum,'(1x,a, 1x,i6, 1x,i6)') 
     :                      ' npts, ln2_npts = ', npts, ln2_npts
      if ( 2.0**ln2_npts /= npts) then
        write(*, '(1x,a, 1x,i6, 1x,a)') 
     :    ' npts = ', npts, ' not a power of 2; QUITTING'
        stop
      else
        npw2 = npts
      end if

! Fill S1 array with zeros
      do i = 1, npw2
        S(i) = 0.0
        FS(i) = cmplx(0.0,0.0)
      end do

! Read in NRATTLE transfer function

      nskip = 2 + nlayers + 8
      call skip(nu_fs, nskip) !should be first line of complex spectrum
         
      do i = 1, npw2/2+1
        read(nu_fs,*) idum, areal_nrattle(i), aimag_nrattle(i), freq(i)
        if (i == 1) then
          write(nu_sum,'(1x,a, 1x, es11.4)') 
     :       ' For '//f_fs(1:nc_f_fs)//', freq(1) = ', freq(1)
        end if
        if (i == npw2/2 + 1) then
          write(nu_sum,'(1x,a, 
     :                   1x,i6, 
     :                   1x,a, 1x,es11.4)')
     :      ' For '//f_fs(1:nc_f_fs)//', freq(',
     :           npw2/2+1,
     :                     ') = ', freq(npw2/2+1)
        end if          
      end do

! Change NRATTLE transfer function to Complex and apply shift operator
! Note the conjg operator was used to correct the output being time reversed.
! I did a number of runs, trying different isign for FORK, etc, before 
! finding that this worked.  I am not sure why, but it probably has to do with the
! sign assumptions in NRATTLE.
      do i =1, npw2/2+1
       FS(i)=cexp(cmplx(0.0,twopi*freq(i)*tshift))*
     :        conjg(CMPLX(areal_nrattle(i),aimag_nrattle(i)))
      end do

! Fill array from Nyquist to NPW2
      do i = NPW2/2 +2, NPW2
        FS(i) = CONJG(FS(NPW2+2-i))
      end do

       
! Change back to time domain
      call fork(NPW2,FS,-1.0)   
!DEBUG
      write(nu_sum,'(1x,a)') 'index, real(FS), imag(FS)'
      do i = 1, npw2
        write(nu_sum,'(1x,i6, 1x,es11.4, 1x,es11.4)') 
     :      i, real(FS(i)), aimag(FS(i))
      end do
!DEBUG
      sps = 2.0*fhigh  
      f2t_factor = sps/sqrt(real(npw2))
      do i=1,NPW2
        S(i)=f2t_factor*real(FS(i))
      end do

      int_null = -32768
      do i = 1, 48
        int_head(i) = int_null
      end do
 
      real_null = 1.7e+38
      do i = 1, 50
        real_head(i) = real_null
      end do

      do i = 2, 11
        char_head(i) = '*'
      end do

      ncomments_in = 0
      ncomments = ncomments_in + 1
      comments(ncomments) = '|'
      ncomments = ncomments + 1
      comments(ncomments)= 
     :      '| Output of program Impulse_Response_From_Nrattle_Output'
      ncomments = ncomments + 1
      comments(ncomments)= '| File with complex spectrum from '//
     :                           'Nrattle:  '
      ncomments = ncomments + 1
      comments(ncomments)= '| '//f_fs(1:nc_f_fs)
      int_head(16)=ncomments

      int_head(17) = npw2

      real_head(2) = sps
      write(nu_sum, '(1x,a, 1x,es11.4)')
     :      ' For '//f_out(1:nc_f_out)//', sps = ', sps

      itype = 0
      smc_precision = ' '
      smc_precision = 'HIGH'
      call trim_c(smc_precision, nc_smc_precision)
      call SMCwrite(f_out(1:nc_f_out),
     :                smc_precision(1:nc_smc_precision),
     :                S, char_head, itype, 
     :                int_head, real_head,comments)

 
      Close(nu_out)
      Close(nu_sum)
 
      Stop
      End

      include '\forprogs\distaz.for'
      include '\forprogs\get_lun.for'
      include '\forprogs\trim_c.for'
      include '\forprogs\skip.for'
      include '\forprogs\mnmaxidx.for'
      include '\forprogs\smcread.for'
      include '\forprogs\smcwrite.for'
      include '\forprogs\fork.for'
      include '\forprogs\skipcmnt.for'
      

! ----------------------------- BEGIN FORK --------------------------
!      SUBROUTINE FORK(LX,CX,SIGNI)
! FAST FOURIER                                  2/15/69
!                          LX
!    CX(K) = SQRT(1.0/LX)* SUM (CX(J)*EXP(2*PI*SIGNI*I*(J-1)*(K-1)/LX))
!                          J=1                        FOR K=1,2,...,LX
!
!  THE SCALING BETWEEN FFT AND EQUIVALENT CONTINUUM OUTPUTS
!  IS AS FOLLOWS.
!
!
!     GOING FROM TIME TO FREQUENCY:
!             F(W)=DT*SQRT(LX)*CX(K)
!
!                  WHERE W(K)=2.0*PI*(K-1)/(LX*DT)
!
!
!     GOING FROM FREQUENCY TO TIME, WHERE THE FREQUENCY
!     SPECTRUM IS GIVEN BY THE DIGITIZED CONTINUUM SPECTRUM:
!
!             F(T)=SQRT(LX)/(LX*DT)*CX(K)
!                  WHERE T(K)=(K-1)*DT
!
!
!  THE RESULT OF THE SEQUENCE...TIME TO FREQUENCY,POSSIBLE MODIFICATIONS
!  OF THE SPECTRUM (FOR FILTERING,ETC.), BACK TO TIME...
!  REQUIRES NO SCALING.
!
!
!  THIS VERSION HAS A SLIGHT MODIFICATION TO SAVE SOME TIME...
!  IT TAKES THE FACTOR 3.1415926*SIGNI/L OUTSIDE A DO LOOP (D.BOORE 12/8
!  FOLLOWING A SUGGESTION BY HENRY SWANGER).
!

! Some brief notes on usage:

! Time to frequency:
! In calling routine,
! 
!       do i = 1, lx
!         cx(i) = CMPLX(y(i), 0.0)
!       end do
!  where y(i) is the time series and lx is a power of 2

! Frequency is given by freq(i) = float(i-1) * df, i = 1, lx/2
! where                     df  = 1.0/(float(lx)*dt)

! 
!  After calling Fork with the complex array specified above, the following 
! symmetries exist:
! 
!        cx(1)        = dc value (f = 0 * df, where df = 1.0/(lx*dt))
!        cx(lx/2 + 1) = value at Nyquist (f = (lx/2+1-1)*df = 1.0/(2*dt))
!        cx(lx)       = CONJG(cx(2))
!        cx(lx-1)     = CONJG(cx(3))
!         |           =      |
!        cx(lx-i+2)   = CONJG(cx(i))
!         |           =      |
!        cx(lx/2+2)   = CONJG(cx(lx/2))
! 
! where "CONJG" is the Fortran complex conjugate intrinsic function
! 
! This symmetry MUST be preserved if modifications are made in the frequency 
! domain and another call to Fork (with a different sign for signi) is used
! to go back to the time domain.  If the symmetry is not preserved, then the
! time domain array will have nonzero imaginary components.  There is one case
! where advantage can be taken of this, and that is to find the Hilbert 
! transform and the window of a time series with only two calls to Fork (there 
! is a short note in BSSA {GET REFERENCE} discussing this trick, which amounts 
! to zeroing out the last half of the array and multiplying all but the dc and 
! Nyquist values by 2.0; in the time domain, REAL(cx(i)) and AIMAG(cx(i)) 
! contain the filtered (if a filter was applied) and Hilbert transform of the 
! filtered time series, respectively, while CABS(cx(i)) and ATAN2(AIMAG(cx(i)), 
! REAL(cx(i))) are the window and instantaneous phase of the filtered time 
! series, respectively.
! 
! The easiest way to ensure the proper symmetry is to zero out the
! last half of the array (as discussed above), but the following is what
! I usually use:  
! modify (filter) only half
! of the cx array:
! 
!       do i = 1, lx/2
!         cx(i) = filter(i)*cx(i)
!       end do
! 
! where "filter(i)" is a possibly complex filter function (and recall that 
! the frequency corresponding to i is f = float(i-1)*df).  After this, fill out
! the last half of the array using
!       
!       do i = lx/2+2, lx
!         cx(i) = CONJG(cx(lx+2-j))
!       end do
! 
! Note that nothing is done with the Nyquist value.  I assume (but am not sure!)
! that this value should be 0.0
! 
! Dates: xx/xx/xxxx - Written by Norm Brenner(?), Jon Claerbout(?)
!        12/21/2000 - Replaced hardwired value of pi with pi evaluated here,
!                     and added comments regarding usage.  Also deleted
!                     dimension specification of cx(lx) and replace it with
!                     cx(*) in the type specification statement.  I also
!                     cleaned up the formatting of the subroutine.
  
