      Program Layr2Plt_Col2File

! Reads col file made by layr2plt for many files and writes
! output into a single file, for import into CoPlot
  
! Reads info from a control file; here is an example:

!!Control file for layr2plt_col2file
!! Revision of program involving a change in the control file on this date:
!   12/15/14
!!Name of the output file:
! vel_slowness_passive_sources.col
!!Number of header lines to skip:
! 3
!!ncols2read, ncol4D, ncol4Vs, ncol4S:
! 3 1 2 3
!!List of input files
!ccoc_avg_vel_1m_using_vtt_for_gibbs_holzer_layered_model.col
!asten_pkdec2_s_thick_layered_model.col
!asten_pkdec3_s_thick_layered_model.col
!asten_pkapr04e_s_thick_layered_model.col
!lang_s_layered_model.col
!lang_oct04_vs_d2b_layered_model.col
!hartzell_spac_vs_layered_model.col
!hartzell_fk_vs_thick_layered_model.col
!stephenson_remi_middle_vs_d2b_orig_layered_model.col
!stephenson_remi_middle_vs_d2b_mar05_layered_model.col
!jlccoc_layered_model.col
!stop
  
! Dates: 05/26/06 - Written by D. Boore, patterned after vel2tt_col2file
!        12/15/14 - Add date of controlf file change 
!                   and number of header lines to skip.
!                 - Add input file names to output
 
      character f_out*120, f_col(100)*120, 
     :          buf*100, buf4*4 
      character cmnts2skip(50)*80, type*1
      character colhead1(100)*10, colhead2(100)*10, colhead3(100)*10
      
      real work(10), coldata(1000, 100, 3)
      
      integer nc_f_col(100)

      character version_ctl*8, version_in*30

      character f_ctl*80
      logical f_exist, susp_log
      
      coldata = -999.0
      
      f_exist = .false.
      do while (.not. f_exist)
        f_ctl = ' '
        write(*, '(a\)') 
     :    ' Enter name of control file (cr = layr2plt_col2file.ctl; '//
     :                                 'ctl-brk to quit): '
        read(*, '(a)') f_ctl
        if (f_ctl .eq. ' ') f_ctl = 'layr2plt_col2file.ctl'
        call trim_c(f_ctl, nc_f_ctl)
        inquire(file=f_ctl(1:nc_f_ctl), exist=f_exist)
        if (.not. f_exist) then
          write(*,'(a)') ' ******* FILE DOES NOT EXIST ******* '
        end if
      end do

      call get_lun(nu_ctl)
      open(unit=nu_ctl,file=f_ctl(1:nc_f_ctl),status='unknown')

!Check version of control file
      call skipcmnt(nu_ctl, cmnts2skip, nc_cmnts2skip)
      version_in = ' '
      read(nu_ctl,'(a)') version_in
      call trim_c(version_in,nc_version_in)
      
      version_ctl = ' '
      version_ctl = '12/15/14'
      call trim_c(version_ctl,nc_version_ctl)

      if (version_ctl(1:nc_version_ctl) .ne. 
     :    version_in(1:nc_version_in)) then
        write(*,'(a)') 
     :     ' The control file has the wrong version date; STOP!'
        write(*,'(a)') 
     :     ' The correct date is '//version_ctl(1:nc_version_ctl)
        close(nu_ctl)
        stop
      end if
      
  
! Get output file name:
      call skipcmnt(nu_ctl, cmnts2skip, nc_cmnts2skip)
      read(nu_ctl, '(a)') f_out
      call trim_c(f_out,nc_f_out)
      call get_lun(nu_out)
      open(unit=nu_out,file=f_out(1:nc_f_out),status='unknown')
  
! Get number of header lines to skip:
      call skipcmnt(nu_ctl, cmnts2skip, nc_cmnts2skip)
      read(nu_ctl,*) nheaders2skip
 
! Get column numbers with values:
      call skipcmnt(nu_ctl, cmnts2skip, nc_cmnts2skip)
      read(nu_ctl,*) ncols2read, ncol4D, ncol4Vs, ncol4S

! Read and process files:
      call skipcmnt(nu_ctl, cmnts2skip, nc_cmnts2skip)

      nfiles = 0
      nd_max = 0

1000  continue

      buf = ' '
      read(nu_ctl,'(a)') buf
      call trim_c(buf, nc_buf)
      buf4 = ' '
      buf4 = buf(1:4)
      call upstr(buf4)
      if (buf4 .eq. 'STOP') then
        close(nu_ctl)
        go to 6969
      end if     

      f_col(nfiles + 1) = ' '
      f_col(nfiles + 1) = buf(1:nc_buf)
      call trim_c(f_col(nfiles + 1), nc_f_col(nfiles + 1))
      f_exist = .false.
      inquire(file=f_col(nfiles + 1)(1:nc_f_col(nfiles + 1)),
     :          exist=f_exist)
      if (.not. f_exist) then
         write(*,'(a)') 
     :  ' ******* FILE '//f_col(nfiles + 1)(1:nc_f_col(nfiles + 1))//
     :  ' DOES NOT EXIST ******* '
         write(*,'(a)') ' Skipping to next file'
         go to 1000
      end if
      
      nfiles = nfiles + 1
 

      call get_lun(nu_col)
      open(nu_col,file=f_col(nfiles)(1:nc_f_col(nfiles)),
     :      status='unknown')

      call skip(nu_col, nheaders2skip)
      
      colhead1(nfiles) = ' '
      colhead1(nfiles)(1:1) = 'd'
      write(colhead1(nfiles)(2:10),'(a)') f_col(nfiles)(1:9) 

      colhead2(nfiles) = ' '
      colhead2(nfiles)(1:1) = 'v'
      write(colhead2(nfiles)(2:10),'(a)') f_col(nfiles)(1:9) 

      colhead3(nfiles) = ' '
      colhead3(nfiles)(1:1) = 's'
      write(colhead3(nfiles)(2:10),'(a)') f_col(nfiles)(1:9) 

      nd_out = 0      
      ncols = ncols2read
3333  continue
        read(nu_col,*, end=4444)  (work(j), j = 1, ncols)
        nd_out = nd_out + 1
        coldata(nd_out,nfiles,1) = work(ncol4D)
        coldata(nd_out,nfiles,2) = work(ncol4Vs)      
        coldata(nd_out,nfiles,3) = work(ncol4S)      
       
        if(nd_max .lt. nd_out) nd_max = nd_out
        
      go to 3333
4444  continue      
      
       
      go to 1000
 
6969  continue

      do i = 1, nfiles
        write(nu_out,'(a)') '! Input file: '//
     :       f_col(i)(1:nc_f_col(i))
      end do

      write(nu_out,'(60(1x,a10, 1x,a10))') 
     :   (colhead1(j), colhead2(j),  colhead3(j), j=1,nfiles)
     
      do i = 1, nd_max
        write(nu_out,'(1p,120(1x,e10.3))') 
     :       (coldata(i,j,1),coldata(i,j,2),coldata(i,j,3),j=1,nfiles) 
      end do
      
      close(nu_out)
      
      stop
      end


      include '\forprogs\get_lun.for'
      include '\forprogs\trim_c.for'
      include '\forprogs\upstr.for'
      include '\forprogs\skip.for'
      include '\forprogs\skipcmnt.for'

      