      PROGRAM ROLL

C  PROGRAM ROLL - SHOVDECK with changes for PDP-VMS system.
C  Haskell-Thomson transfer function for horizontally stratified medium.
C  900 layers max.
C  Original SHOVDECK program from W.JOYNER (MARCH,1981).
C  Checked and made VAX-compatible by Chuck Mueller.
C
C  Haskell's approach: Propagate particle velocity and stress through
C    (m)th layer [from (m-1)st boundary to (m)th boundary].
C  Joyner's approach: Propagate coefficients of up- and downgoing waves
C    across (m)th boundary [from (m+1)st layer to (m)th layer].
C  (I haven't done the math to check this for ROLL - inferred from RATTLE
C    where I have done the math. CSM, 2/83)
C
C  Needs to be redone with Aki-Richards conventions.(?)

*Sample control file:
*!Control file for program ROLL
*!Name of output file with complex spectra:                                     
*    roll_amp.out
*!Name of output file for plots (columns of frequency and response amplitude):  
*    roll_amp.asc
*!Number of frequencies (including f = 0.0) and highest freq. (nfreq, fhigh):   
*   100    30.00
*!Depth at which response is computed (0.0 for free surface):                   
*    0.00
*!angle of incidence: 
*    0.00
*!P or S?
*  S
*!H or V component?
*  H
*!Units of velocity (kps, mps):
* kps
*!Velocity model:
*! Halfspace Vp, Vs and density (1/Q is automatically set to 0.0):
*    5.2 3.00  2.7
*!  For each layer- thickness, Vp, Vs, density, Qp, Qs (if < 1.0, the program assumes
*!  that the value of Q represents 1/Q; note that damping in percent = 100*0.5*1/Q):
*! Use my density--velocity relation if density = 0.0
*    0.008 0.170  0.100  1.9  30.0 17.0
*    0.022 1.080  0.600  2.2  120.0 50.0
*    stop


C  

* Dates: 08/86    - Taught ROLL to skip comment lines at top of input file.
*        01/16/97 - giovanna modified the program for the pc and increased
*                   array dimensions (like 'rattle.for')
*        01/21/96 - i/o changes, asking for file names and writing a file
*                   with fewer header lines for use in COPLOT (to be imported
*                   as a wrapped ASCII file).  
*        09/04/98 - Some IO changes by Dave Boore
*        01/24/08 - Some changes to modernize for latest compiler.
*        01/24/08 - Get input from a control file
!        03/08/10 - Long delayed debugging (based on roll_old.for, which works)
!        03/05/ 12 - More debugging and modernization (removing statement labels, etc)

!WARNING!!! THIS HAS NOT BEEN DEBUGGED YET

      real Z(900),RHO(901)
      complex CI,CA,CB,CIP,CX,CY,CZ,
     *        BN,DN,BNR,DNR,
     *        A(901),A2(901),B(901),B2(901),XLA(901),XLB(901),
     *        T11,T14,T41,T44,T22,T23,T32,T33
      complex Q11,Q12,Q13,Q14,Q21,Q22,Q23,Q24,
     *        Q31,Q32,Q33,Q34,Q41,Q42,Q43,Q44
      complex P11(901),P12(901),P13(901),P14(901),P21(901),P22(901),
     *        P23(901),P24(901),P31(901),P32(901),P33(901),P34(901),
     *        P41(901),P42(901),P43(901),P44(901)
      complex R31,R34,R42,R43,R11,R14,R22,R24,
     *        F1,F2,G11,G21,G12,G22,
     *        RN31,RN34,RN42,RN43,
     *        VV
     
      character comment(20)*80, buf4*4, units_c*10
      character BUF*72,date_c*8,time_c*10,PORS*10,HORV*10
      character f_ctl*80, f_out*80, f_4plt*80, namein*80
      character tempbuf*8, col1*12, col2*12, col3*12, name_plt*12
      
      integer ios
      
      logical f_ctl_exist, specify_dens_coeff, found_zdepth

      CI = (0.0,1.0)
      PI = 4.0*ATAN(1.0)
      iflag = 0
      call date_and_time(date_c,time_c)
 
      specify_dens_coeff = .false.
      
C...Open files.

      f_ctl_exist = .false.
      do while (.not. f_ctl_exist)
        f_ctl = ' '
        write(*, '(a\)') 
     :    ' Enter name of control file ("Enter"=roll.ctl; '//
     :    'ctl-brk to quit): '
        read(*, '(a)') f_ctl
        if (f_ctl  == ' ') f_ctl = 'roll.ctl'
        call trim_c(f_ctl,nc_f_ctl)
        inquire(file=f_ctl(1:nc_f_ctl), exist=f_ctl_exist)
        if (.not. f_ctl_exist) then
          write(*,'(a)') ' ******* FILE DOES NOT EXIST ******* '
        end if
      end do
      
      call get_lun(nu_ctl)
      open(unit=nu_ctl,file=f_ctl(1:nc_f_ctl),status='unknown')

      call skipcmnt(nu_ctl, comment, ncomments)
      
* Get names of output files and open the files:

      f_out = ' '
      read(nu_ctl, '(a)') f_out
      call trim_c(f_out,nc_f_out)
      call get_lun(nu_out)
      open(unit=nu_out, file=f_out(1:nc_f_out), status='unknown')
      
      call skipcmnt(nu_ctl, comment, ncomments)

      f_4plt = ' '
      read(nu_ctl, '(a)') f_4plt
      call trim_c(f_4plt, nc_f_4plt)
      call get_lun(nu_4plt)
      open(unit=nu_4plt, file=f_4plt(1:nc_f_4plt), status='unknown')

      write(nu_out, '(a)') ' Program ROLL'
      write(nu_out, '(a)')  
     :  ' Date: '//date_c(1:4)//'-'//date_c(5:6)//'-'//date_c(7:8)
      write(nu_out, '(a)')  
     : ' Time: '//time_c(1:2)//'h'//time_c(3:4)//'m'//time_c(5:10)//'s'
      write(nu_out,*)
      
      write(nu_4plt, '(a)') ' Program ROLL'
      write(nu_4plt, '(a)')  
     :  ' Date: '//date_c(1:4)//'-'//date_c(5:6)//'-'//date_c(7:8)
      write(nu_4plt, '(a)')  
     : ' Time: '//time_c(1:2)//'h'//time_c(3:4)//'m'//time_c(5:10)//'s'
      write(nu_4plt,*)
      
* Get nfreq and fhigh and convert to quantities used in calculations

      call skipcmnt(nu_ctl, comment, ncomments)

      read(nu_ctl,*) nfreq, fhigh
      inyq = nfreq
      samp = 2*fhigh
      t = 2*(inyq-1)/samp
      delf = 1.0/t

!
! 
!      IPW2 = 2**MX !Number of points in corresponding time series.
!      INYQ = IPW2/2+1
!      T = IPW2/SPS
!      DELF = 1.0/T
      
      write(nu_out,*)

* Get depth at which response is computed:

      call skipcmnt(nu_ctl, comment, ncomments)

      read (nu_ctl,*) zdepth
      write(nu_out,'(a,f10.4)')
     :  ' Depth at which response is computed = ', zdepth
      write(nu_out,*)

* Get angle of incidence

      call skipcmnt(nu_ctl, comment, ncomments)

      read(nu_ctl,*) theta
      write(nu_out,'(a,f7.2)') 
     :  ' Angle of incidence in halfspace ='  ,theta
      write(nu_out,*)

      write(nu_4plt,'(a,f7.2)') 
     :  ' Angle of incidence in halfspace ='  ,theta
      write(nu_4plt,*)

* P or S?
      call skipcmnt(nu_ctl, comment, ncomments)
      PorS = ' '
      read(nu_ctl,'(a)') PorS
      call trim_c(PorS,nc_PorS)
      call upstr(PorS)

      write(nu_out,'(a)') 
     :  ' PorS ='//PorS(1:nc_PorS)
      write(nu_out,*)

      write(nu_4plt,'(a)') 
     :  ' PorS ='//PorS(1:nc_PorS)
      write(nu_4plt,*)

* H or V?
      call skipcmnt(nu_ctl, comment, ncomments)
      HorV = ' '
      read(nu_ctl,'(a)') HorV
      call trim_c(HorV,nc_HorV)

      write(nu_out,'(a)') 
     :  ' HorV ='//HorV(1:nc_HorV)
      write(nu_out,*)

      write(nu_4plt,'(a)') 
     :  ' HorV ='//HorV(1:nc_HorV)
      write(nu_4plt,*)

* Velocity model:

      call skipcmnt(nu_ctl, comment, ncomments)

      units_c = ' '
      read(nu_ctl,'(a)') units_c
      call trim_c(units_c, nc_units_c)

      write(nu_out,'(a)') 
     :  ' Units ='//units_c(1:nc_units_c)
      write(nu_out,*)

      write(nu_4plt,'(a)') 
     :  ' Units ='//units_c(1:nc_units_c)
      write(nu_4plt,*)


* Halfspace properties

      call skipcmnt(nu_ctl, comment, ncomments)

      read(nu_ctl,*) vpn, betan, rhon

      write(nu_out,'(a)') ' Halfspace properties: '
      write(nu_out,'(a, 3(1x,f8.3))') 
     :  ' vpn, betan, rhon =', vpn, betan, rhon
      write(nu_out,*)

      write(nu_4plt,'(a)') ' Halfspace properties: '
      write(nu_4plt,'(a, 3(1x,f8.3))') 
     :  ' vpn, betan, rhon =', vpn, betan, rhon
      write(nu_4plt,*)
      
      if (PorS(1:1) == 'P') rat=sin(pi*theta/180.0)/vpn
      if (PorS(1:1) == 'S') rat=sin(pi*theta/180.0)/betan
      rat2 = rat*rat

    
     
! Layer properties:

      write(nu_out,*)
      write(nu_out,'(
     :               1x,a, 1x,a, 1x,a, 1x,a,
     :               1x,a, 1x,a, 1x,a
     :                                             )') 
     :         'layer', 'thickness', 'vp', 'vs',
     :         'density', 'qp', 'qs'

      call skipcmnt(nu_ctl, comment, ncomments)

      nlay = 0
      zcum = 0.0
      
      loop over model: DO
        buf = ' '
        read(nu_ctl,'(a)', iostat = ios) buf
        if (ios /= 0) then
          EXIT loop over model
        end if
      
        call trim_c(buf,nc_buf)
        buf4 = ' '
        buf4 = buf(1:4)
        call upstr(buf4)
        if (buf4 == 'STOP') then
          EXIT loop over model
        end if
      
        nlay = nlay + 1
        read (buf(1:nc_buf),*) thick,vp,vs,rho(nlay),qp,qs
        if (qp < 1.0) qp = 1.0/qp
        if (qs < 1.0) qs = 1.0/qs
        write (nu_out,*) thick,vp,vs,rho(nlay),qp,qs
        if (rho(nlay) == 0.0) then 
           rho(nlay) = density(vs,
     :                  specify_dens_coeff,
     :                  dens_low, vel_low,
     :                  dens_high, vel_high, dd_dv, units_c)
        end if
        zcum = zcum+thick
        z(nlay) = zcum
        a2(nlay) = vp*vp*(1.0+ci/qp)
        a(nlay) = csqrt(a2(nlay))
        b2(nlay) = vs*vs*(1.0+ci/qs)
        b(nlay) = csqrt(b2(nlay))
        xla(nlay) = csqrt(1.0/a2(nlay)-rat2)
        xlb(nlay) = csqrt(1.0/b2(nlay)-rat2)
      
      END DO loop over model
      
      close (nu_ctl)
 
      write(nu_out,*)
      WRITE (nu_out,90115) PORS(1:nc_PorS)
90115 FORMAT (' Incident wave type= ',A1)
      WRITE (nu_out,90116) NLAY,nfreq,DELF,samp,t,THETA,VPN,BETAN,RHON
90116 FORMAT (' Number of layers=',I10/
     *        ' Number of frequencies=',I10/
     *        ' Delta F (Hz)=',F10.7/
     *        ' Sample rate=',F10.4/
     :        ' length of equivalent time series=',F10.4/
     *        ' Incidence angle (deg)=',F10.4/
     *        ' Substratum P-velocity (km/s)=',F10.4/
     *        ' Substratum S-velocity (km/s)=',F10.4/
     *        ' Substratum density (gm/cc)=',F10.4)
      WRITE (nu_4plt,90123) THETA,vpn,BETAN,RHON
90123 FORMAT (' Incidence angle (deg)=',F10.4/
     *        ' Substratum P and S velocities (km/s)=',2F10.4/
     *        ' Substratum density (gm/cc)=',F10.4)
      write(nu_out,*)
 
      n = nlay+1
      rho(n) = rhon
      a(n) = vpn
      a2(n) = a(n)*a(n)
      b(n) = betan
      b2(n) = b(n)*b(n)
      xla(n) = csqrt(1.0/a2(n)-rat2)
      xlb(n) = csqrt(1.0/b2(n)-rat2)
      p11(n) = 1.0
      p12(n) = 0.0
      p13(n) = 0.0
      p14(n) = 0.0
      p21(n) = 0.0
      p22(n) = 1.0
      p23(n) = 0.0
      p24(n) = 0.0
      p31(n) = 0.0
      p32(n) = 0.0
      p33(n) = 1.0
      p34(n) = 0.0
      p41(n) = 0.0
      p42(n) = 0.0
      p43(n) = 0.0
      p44(n) = 1.0
      r31 = rho(1)*a2(1)*(1.0-2.0*rat2*b2(1))
      r34 = 4.0*rho(1)*rat*xlb(1)*b2(1)*b2(1)
      r42 = -2.0*rho(1)*rat*xla(1)*a2(1)*b2(1)
      r43 = 2.0*rho(1)*b2(1)*b2(1)*(xlb(1)*xlb(1)-rat2)
      rn31 = rho(n)*a2(n)*(1.0-2.0*rat2*b2(n))
      rn34 = 4.0*rho(n)*rat*xlb(n)*b2(n)*b2(n)
      rn42 = -2.0*rho(n)*rat*xla(n)*a2(n)*b2(n)
      rn43 = 2.0*rho(n)*b2(n)*b2(n)*(xlb(n)*xlb(n)-rat2)
      
      if (pors(1:nc_PorS)=='P') then
        bnr = (rn43*rn31+rn34*rn42)/(rn34*rn42-rn43*rn31)
        dnr = 2.0*rn42*rn31/(rn42*rn34-rn31*rn43)
      else if (pors(1:nc_PorS)=='S') then
        bnr = 2.0*rn43*rn34/(rn34*rn42-rn43*rn31)
        dnr = (rn42*rn34+rn31*rn43)/(rn42*rn34-rn31*rn43)
      else
        print *, ' ERROR: pors = '//pors(1:nc_PorS)//
     :           '; not "P" or "S". QUIT!!'
        close(nu_out)
        close(nu_4plt)
        stop
      endif

      write (nu_out,*)  
      write (nu_out,90310) HorV(1:nc_HorV),zdepth
90310 format (' type of calculation= ',a1/
     *        ' depth of calculation=',f10.4)
      write (nu_out,'(1x,a)') ' '
      write (nu_out,90315)
90315 format (t4,'nfreq', t14,'real(s)', t26,'imag(s)',
     :        t41,'freq', t50,'cabs(s)' )
      
      write (nu_4plt,*)
      tempbuf = ' '
      tempbuf = f_4plt(1:index(f_4plt,'.')-1)
      call trim_c(tempbuf,nchar_tb)
      col1 = ' '
      col1(9-nchar_tb:8) = tempbuf(1:nchar_tb)
      col1(9:12) = '-frq'
      col2 = ' '
      col2(9-nchar_tb:8) = tempbuf(1:nchar_tb)
      col2(9:12) = '-per'
      col3 = ' '
      col3(9-nchar_tb:8) = tempbuf(1:nchar_tb)
      col3(9:12) = '-amp'
      write(nu_4plt, '(1x,a, 1x,a, 1x,a)') col1, col2, col3
 

c**********
c
c  loop over frequency. index = ifrq
c
c**********
      loop over frequency: DO ifrq=1,inyq
      
         if (ifrq==1) then
           f = 0.0
           per = 1.0e+06
           vv = (1.0,0.0)
           write (nu_out,'(t4,i5, 4(1x,es11.4))') ifrq,vv, f, cabs(vv) 
           WRITE (nu_4plt,'(1x,f12.7, 2(1x,es12.4))') f, per, cabs(vv)
           cycle loop over frequency
         endif
         
         f=(ifrq-1)*delf
         p=2.0*pi*f
         cip=ci*p
         
c**********
c
c  loop over the number of layers (propagate). indices = i,k
c
c**********
         do itmp=1,nlay
            i=nlay+1-itmp
            k=i+1
            cx=rho(k)/rho(i)
            cy=(rho(k)*b2(k)-rho(i)*b2(i))/rho(i)
            cz=2.0*rat2*cy
            t11=(a2(k)/a2(i))*(cx-cz)
            t14=(4.0*rat*xlb(k)*b2(k)/a2(i))*cy
            t41=((rat*a2(k))/(2.0*xlb(i)*b2(i)))*(cx-1.0-cz)
            t44=((xlb(k)*b2(k))/(xlb(i)*b2(i)))*(1.0+cz)
            t22=((xla(k)*a2(k))/(xla(i)*a2(i)))*(1.0+cz)
            t23=((2.0*rat*b2(k))/(xla(i)*a2(i)))*(1.0-cx+cz)
            t32=((-rat*xla(k)*a2(k))/b2(i))*cy
            t33=(b2(k)/b2(i))*(cx-cz)

            cx=cip*z(i)
            q11=0.5*(t11+t22)*cexp( cx*(xla(k)-xla(i)))
            q12=0.5*(t11-t22)*cexp(-cx*(xla(k)+xla(i)))
            q13=0.5*(t14+t23)*cexp( cx*(xlb(k)-xla(i)))
            q14=0.5*(t23-t14)*cexp(-cx*(xlb(k)+xla(i)))
            q21=0.5*(t11-t22)*cexp( cx*(xla(i)+xla(k)))
            q22=0.5*(t11+t22)*cexp( cx*(xla(i)-xla(k)))
            q23=0.5*(t14-t23)*cexp( cx*(xla(i)+xlb(k)))
            q24=0.5*(-t14-t23)*cexp( cx*(xla(i)-xlb(k)))
            q31=0.5*(t32+t41)*cexp( cx*(xla(k)-xlb(i)))
            q32=0.5*(t41-t32)*cexp(-cx*(xla(k)+xlb(i)))
            q33=0.5*(t33+t44)*cexp( cx*(xlb(k)-xlb(i)))
            q34=0.5*(t33-t44)*cexp(-cx*(xlb(k)+xlb(i)))
            q41=0.5*(t32-t41)*cexp( cx*(xlb(i)+xla(k)))
            q42=0.5*(-t32-t41)*cexp( cx*(xlb(i)-xla(k)))
            q43=0.5*(t33-t44)*cexp( cx*(xlb(i)+xlb(k)))
            q44=0.5*(t33+t44)*cexp( cx*(xlb(i)-xlb(k)))
            p11(i)=q11*p11(k)+q12*p21(k)+q13*p31(k)+q14*p41(k)
            p12(i)=q11*p12(k)+q12*p22(k)+q13*p32(k)+q14*p42(k)
            p13(i)=q11*p13(k)+q12*p23(k)+q13*p33(k)+q14*p43(k)
            p14(i)=q11*p14(k)+q12*p24(k)+q13*p34(k)+q14*p44(k)
            p21(i)=q21*p11(k)+q22*p21(k)+q23*p31(k)+q24*p41(k)
            p22(i)=q21*p12(k)+q22*p22(k)+q23*p32(k)+q24*p42(k)
            p23(i)=q21*p13(k)+q22*p23(k)+q23*p33(k)+q24*p43(k)
            p24(i)=q21*p14(k)+q22*p24(k)+q23*p34(k)+q24*p44(k)
            p31(i)=q31*p11(k)+q32*p21(k)+q33*p31(k)+q34*p41(k)
            p32(i)=q31*p12(k)+q32*p22(k)+q33*p32(k)+q34*p42(k)
            p33(i)=q31*p13(k)+q32*p23(k)+q33*p33(k)+q34*p43(k)
            p34(i)=q31*p14(k)+q32*p24(k)+q33*p34(k)+q34*p44(k)
            p41(i)=q41*p11(k)+q42*p21(k)+q43*p31(k)+q44*p41(k)
            p42(i)=q41*p12(k)+q42*p22(k)+q43*p32(k)+q44*p42(k)
            p43(i)=q41*p13(k)+q42*p23(k)+q43*p33(k)+q44*p43(k)
            p44(i)=q41*p14(k)+q42*p24(k)+q43*p34(k)+q44*p44(k)
         end do

c*******************
         g11=r31*(p12(1)+p22(1))+r34*(p32(1)-p42(1))
         g21=r42*(p12(1)-p22(1))+r43*(p32(1)+p42(1))
         g12=r31*(p14(1)+p24(1))+r34*(p34(1)-p44(1))
         g22=r42*(p14(1)-p24(1))+r43*(p34(1)+p44(1))
         
         test=cabs(g22*g11-g12*g21)
         if(test == 0.0) then     !matrix singular
           print *,' For frequency = ', f, 
     :         ' the matrix is singular; QUIT!!'
2500     write (nu_out,92500) f
         write (nu_4plt,92500) f
92500    format(f8.2,10x,'matrix singular - point on rayleigh-wave disper
     :           sion curve')
           stop
         end if

         if (pors(1:nc_PorS)=='P') then
           f1=r31*(p11(1)+p21(1))+r34*(p31(1)-p41(1))
           f2=r42*(p11(1)-p21(1))+r43*(p31(1)+p41(1))
         elseif (pors(1:nc_PorS)=='S') then
           f1=r31*(p13(1)+p23(1))+r34*(p33(1)-p43(1))
           f2=r42*(p13(1)-p23(1))+r43*(p33(1)+p43(1))
         else
           print *, ' ERROR: pors = '//pors(1:nc_PorS)//
     :           '; not "P" or "S". QUIT!!'
           close(nu_out)
           close(nu_4plt)
           stop
         endif
         bn=(g12*f2-g22*f1)/(g22*g11-g12*g21)
         dn=(g11*f2-g21*f1)/(g21*g12-g11*g22)

         found_zdepth = .false.
         do i4loop=1,nlay
           if(zdepth <= z(i4loop)) then
             found_zdepth = .true.
             exit
           end if
         end do

         if (found_zdepth) then
           i = i4loop
         else
           i = n
         end if
         
         r11 = -rat*a2(i)
         r14 = 2.0*xlb(i)*b2(i)
         r22 = xla(i)*a2(i)
         r23 = 2.0*rat*b2(i)
         ca = cip*xla(i)*zdepth
         cb = cip*xlb(i)*zdepth

         if (pors(1:nc_PorS) == 'P') then
           if (horv(1:nc_HorV) == 'V') then
             vv = (r22*((p11(i)+p12(i)*bn+p14(i)*dn)*cexp(ca)-
     :            (p21(i)+p22(i)*bn+p24(i)*dn)*cexp(-ca))+
     :            r23*((p31(i)+p32(i)*bn+p34(i)*dn)*cexp(cb)+
     :            (p41(i)+p42(i)*bn+p44(i)*dn)*cexp(-cb)))/
     :            (xla(n)*a2(n)*(1.0-bnr)+2.0*rat*b2(n)*dnr)
           else if (horv(1:nc_HorV)=='H') then
             if (theta==0.0) then
               vv = 0.0
             else
               vv = (r11*((p11(i)+p12(i)*bn+p14(i)*dn)*cexp(ca)+
     :              (p21(i)+p22(i)*bn+p24(i)*dn)*cexp(-ca))+
     :              r14*((p31(i)+p32(i)*bn+p34(i)*dn)*cexp(cb)-
     :              (p41(i)+p42(i)*bn+p44(i)*dn)*cexp(-cb)))/
     :              (-rat*a2(n)*(1.0+bnr)-2.0*xlb(n)*b2(n)*dnr)
             endif
           else
             print *, ' ERROR: horv = '//horv(1:nc_HorV)//
     :           '; not "H" or "V". QUIT!!'
             close(nu_out)
             close(nu_4plt)
             stop
           endif
         else if (pors(1:nc_PorS)=='S') then
           if (horv(1:nc_HorV)=='V') then
             if (theta==0.0) then
               vv = 0.0
             else
               vv = (r22*((p12(i)*bn+p13(i)+p14(i)*dn)*cexp(ca)-
     :              (p22(i)*bn+p23(i)+p24(i)*dn)*cexp(-ca))+
     :              r23*((p32(i)*bn+p33(i)+p34(i)*dn)*cexp(cb)+
     :              (p42(i)*bn+p43(i)+p44(i)*dn)*cexp(-cb)))/
     :              (-xla(n)*a2(n)*bnr+2.0*rat*b2(n)*(1.0+dnr))
             endif
           elseif (horv(1:nc_HorV)=='H') then
             vv = (r11*((p12(i)*bn+p13(i)+p14(i)*dn)*cexp(ca)+
     :            (p22(i)*bn+p23(i)+p24(i)*dn)*cexp(-ca))+
     :            r14*((p32(i)*bn+p33(i)+p34(i)*dn)*cexp(cb)-
     :            (p42(i)*bn+p43(i)+p44(i)*dn)*cexp(-cb)))/
     :            (-rat*a2(n)*bnr+2.0*xlb(n)*b2(n)*(1.0-dnr))
           else
             print *, ' ERROR: horv = '//horv(1:nc_HorV)//
     :           '; not "H" or "V". QUIT!!'
             close(nu_out)
             close(nu_4plt)
             stop
           endif
         else
           print *, ' ERROR: pors = '//pors(1:nc_PorS)//
     :           '; not "P" or "S". QUIT!!'
           close(nu_out)
           close(nu_4plt)
           stop
         endif

         write (nu_out,'(t4,i5, 4(1x,es11.4))') ifrq,vv, f, cabs(vv) 
         if (f == 0.0) then
           per = 1.0e+06
         else
           per = 1.0/f
         end if
         WRITE (nu_4plt,'(1x,f12.7, 2(1x,es12.4))') f, per, cabs(vv)
      
      END DO loop over frequency

      close (unit=nu_out)
      close(unit=nu_4plt)

      end

      include 'site_amp_util_subs.for'
  
!      include 'nrtlsubs.for'
 
  