
! --------------------- BEGIN UPSTR ----------------------------------
      Subroutine UPSTR ( text )
! Converts character string in TEXT to uppercase
! Dates: 03/12/96 - Written by Larry Baker
!        04/28/15 - Replaced comment characters * or C with ! (The Fortran 95 standard)

!
      Implicit   None
!
      Character  text*(*)
!
      Integer    j
      Character  ch
!
      Do 1000 j = 1,LEN(text)
         ch = text(j:j)
         If ( LGE(ch,'a') .and. LLE(ch,'z') ) Then
            text(j:j) = CHAR ( ICHAR(ch) - ICHAR('a') + ICHAR('A') )
         End If
 1000    Continue
!
      Return
      End
! --------------------- END UPSTR ----------------------------------


! --------------------------- BEGIN TRIM_C -----------------------
      subroutine trim_c(cstr, nchar)

! strips leading and trailing blanks from cstr, returning the
! result in cstr, which is now nchar characters in length

! Strip off tabs also.

! Here is a sample use in constructing a column header, filled out with 
! periods:

!* Read idtag:
!        idtag = ' '
!        read(nu_in, '(1x,a)') idtag
!        call trim_c(idtag, nc_id)
!* Set up the column headings:
!        colhead = ' '
!        colhead = idtag(1:nc_id)//'......' ! nc_id + 6 > length of colhead

! Dates: 12/23/97 - written by D. Boore
!        12/08/00 - pad with trailing blanks.  Otherwise some comparisons
!                   of the trimmed character string with other strings
!                   can be in error because the original characters are left
!                   behind after shifting.  For example, here is a string
!                   before and after shifting, using the old version:
!                      col:12345
!                           MTWH  before
!                          MTWHH  after (but nc = 4).
!        03/21/01 - Check for a zero length input string
!        11/09/01 - Change check for zero length string to check for all blanks
!        10/19/09 - Strip off tabs
!        04/28/15 - Replaced comment characters * or C with ! (The Fortran 95 standard)
!        02/14/18 - Use "len_trim" to check for cstr being all blank

      character cstr*(*)

      nend = len_trim(cstr)

      if(nend == 0) then
        nchar = 0
        return
      end if

! Replace tabs with blanks:

      do i = 1, nend
        if(ichar(cstr(i:i)) == 9) then
           cstr(i:i) = ' '
        end if
      end do



!      if(nend == 0) then
!        nchar = 0
!        return
!      end if

      do i = nend, 1, -1
        if (cstr(i:i) /= ' ') then
           nchar2 = i
           goto 10
        end if
      end do

10    continue

      do j = 1, nchar2
        if (cstr(j:j) /= ' ') then
          nchar1 = j
          goto 20
        end if
      end do

20    continue
   
      nchar = nchar2 - nchar1 + 1
      cstr(1:nchar) = cstr(nchar1: nchar2)
      if (nchar < nend) then
        do i = nchar+1, nend
          cstr(i:i) = ' '
        end do
      end if

      return
      end
! --------------------------- END TRIM_C -----------------------

! --------------------------- BEGIN GET_LUN ----------------
      subroutine get_lun(lun)

! Finds a logical unit number not in use; returns
! -1 if it cannot find one.

! Dates -- 05/19/98 - Written by D. Boore, following
!                     Larry Baker's suggestion
!        04/28/15 - Replaced comment characters * or C with ! (The Fortran 95 standard)

      logical isopen
      do i = 99,10,-1
        inquire (unit=i, opened=isopen)
        if(.not.isopen) then
          lun = i
          return
        end if
      end do
      lun = -1

      return
      end
! --------------------------- END GET_LUN ----------------
     
! ---------------------------- BEGIN LEN_TRIM ----------------
      function len_trim(text)

! returns length of TEXT without trailing blanks

! Dates: 03/14/96 - Written by D. Boore
!        04/29/15 - Replaced comment characters * or C with ! (The Fortran 95 standard)

      character text*(*)
      len_trim = len(text)
      if(text .eq. ' ') then
        len_trim = 0
      else
        do while(text(len_trim:len_trim) .eq. ' ')
           len_trim = len_trim -1
        end do
      end if
      return
      end
! ---------------------------- END LEN_TRIM ----------------

! ---------------------- BEGIN SKIP -------------------
      subroutine SKIP(lunit, nlines)
        if (nlines .lt. 1) then
          return
        else
          do i = 1, nlines
             read(lunit, *)
          end do
          return
        end if
      end
! ---------------------- END SKIP -------------------
! ------------------------------------------------------------------ skipcmnt
      subroutine skipcmnt(nu, comment, ncomments)

! Skip text comments in the file attached to unit nu, but save skipped 
! comments in character array comment.  Skip at least one line, and more as 
! long as the lines are preceded by "|" or "!".

! Dates: 04/16/01 - Written by D. Boore
!        12/07/01 - Added check for eof
!        11/04/03 - Use trim_c to trim possible leading blank
!        02/03/07 - Initialize comments to blank
!        04/28/15 - Replaced comment characters * or C with ! (The Fortran 95 standard)

      character comment(*)*(*), buf*80

      ncomments = 0
100   buf = ' '
      read (nu,'(a)',end=999) buf
      call trim_c(buf,nc_buf)
      if (buf(1:1) .eq.'!' .or. buf(1:1) .eq.'|' .or. 
     :                     ncomments + 1 .eq. 1) then
        ncomments = ncomments + 1
        comment(ncomments) = ' '
        comment(ncomments) = buf(1:nc_buf)
        goto 100
      else 
        backspace nu
      end if

999   continue
 
      return
      end
! ------------------------------------------------------------------ skipcmnt
! ---------------------- BEGIN MNMAX ----------------------
      subroutine mnmax(a,nstrt,nstop,ninc,amin,amax) 
!
! Dates: 09/07/84 - Written by D. Boore
!        11/02/12 - replace obsolete if statement 
! 
!
      dimension a(*)
      
      amax = a(nstrt)
      amin=amax 
      
      do i=nstrt,nstop,ninc

        if ( a(i) < amin) then
          amin=a(i)
        else if ( a(i) > amax) then
          amax=a(i) 
        end if
  
      end do
      
      return                  
      end                     
! ---------------------- END MNMAX ----------------------

!* --------------------- BEGIN LOCATE -----------------
      SUBROUTINE locate(xx,n,x,j)
      
! Comments added by D. Boore on 26feb2010:
!  finds j such that xx(j) < x <= xx(j+1)
!  EXCEPT if x = xx(1), then j = 1 (logically it would be 0 from
!  the above relation, but this is the same returned value of j
!  for a point out of range).
!  Also, if x = xx(n), j = n-1, which is OK
!  Note that j = 0 or j = n indicates that x is out of range.
!
! See the program test_locate.for to test this routine.

! Dates: 04/28/15 - Replaced comment characters * or C with ! (The Fortran 95 standard)

      INTEGER j,n
      REAL x,xx(n)
      INTEGER jl,jm,ju
      jl=0
      ju=n+1
10    if(ju-jl.gt.1)then
        jm=(ju+jl)/2
        if((xx(n).ge.xx(1)).eqv.(x.ge.xx(jm)))then
          jl=jm
        else
          ju=jm
        endif
      goto 10
      endif
      if(x.eq.xx(1))then
        j=1
      else if(x.eq.xx(n))then
        j=n-1
      else
        j=jl
      endif
      return
      END
!* --------------------- END LOCATE -----------------

! ------------------ BEGIN SORT --------------------------
!
! Dates: 12/01/15 - Replace "pause" with "write", and stop the program

      SUBROUTINE sort(n,arr)
      INTEGER n,M,NSTACK
      REAL arr(n)
      PARAMETER (M=7,NSTACK=50)
      INTEGER i,ir,j,jstack,k,l,istack(NSTACK)
      REAL a,temp
      jstack=0
      l=1
      ir=n
1     if(ir-l.lt.M)then
        do 12 j=l+1,ir
          a=arr(j)
          do 11 i=j-1,l,-1
            if(arr(i).le.a)goto 2
            arr(i+1)=arr(i)
11        continue
          i=l-1
2         arr(i+1)=a
12      continue
        if(jstack.eq.0)return
        ir=istack(jstack)
        l=istack(jstack-1)
        jstack=jstack-2
      else
        k=(l+ir)/2
        temp=arr(k)
        arr(k)=arr(l+1)
        arr(l+1)=temp
        if(arr(l).gt.arr(ir))then
          temp=arr(l)
          arr(l)=arr(ir)
          arr(ir)=temp
        endif
        if(arr(l+1).gt.arr(ir))then
          temp=arr(l+1)
          arr(l+1)=arr(ir)
          arr(ir)=temp
        endif
        if(arr(l).gt.arr(l+1))then
          temp=arr(l)
          arr(l)=arr(l+1)
          arr(l+1)=temp
        endif
        i=l+1
        j=ir
        a=arr(l+1)
3       continue
          i=i+1
        if(arr(i).lt.a)goto 3
4       continue
          j=j-1
        if(arr(j).gt.a)goto 4
        if(j.lt.i)goto 5
        temp=arr(i)
        arr(i)=arr(j)
        arr(j)=temp
        goto 3
5       arr(l+1)=arr(j)
        arr(j)=a
        jstack=jstack+2
        if(jstack.gt.NSTACK)then
          write(*,'(1x,a)') 'NSTACK too small in sort; STOPPING'
          stop
        end if
!        if(jstack.gt.NSTACK)pause 'NSTACK too small in sort'
        if(ir-i+1.ge.j-l)then
          istack(jstack)=ir
          istack(jstack-1)=i
          ir=j-1
        else
          istack(jstack)=j-1
          istack(jstack-1)=l
          l=i
        endif
      endif
      goto 1
      END
! ------------------ END SORT --------------------------
! -------------------- BEGIN TT4vert -------------------------
      function tt4vert(z, d2bot, vel, nlayers)

! Dates: 03/29/99 - Written by D. Boore
!        05/09/03 - Extrapolate if z > deepest layer
!        04/29/15 - Replaced comment characters * or C with ! (The Fortran 95 standard)

      real d2bot(0:*), vel(*)

      tt4vert = 0.0
      do i = 1, nlayers
        if( z .gt. d2bot(i) ) then
          tt4vert = tt4vert + (d2bot(i)-d2bot(i-1))/vel(i)
        else
          z_in_layer = z - d2bot(i-1)  ! for this to work, need d2bot(0)=0.0
          tt4vert = tt4vert + z_in_layer/vel(i)
          go to 578
        end if
      end do
      if (z .gt. d2bot(nlayers)) then
          tt4vert = tt4vert + (z - d2bot(nlayers))/vel(nlayers)
      end if
578   continue

      return
      end
! -------------------- END TT4vert -------------------------
! --------------------------- BEGIN DENSITY ----------------------------
      function density(vels,
     :                 specify_dens_coeff,
     :                 dens_low, vel_low,
     :                 dens_high, vel_high, dd_dv, units_c)

! Assigns a density based on shear velocity.  If specify_dens_coeff = .true.,
! then a straight line is fit between the coefficients (and density = dens_low
! for vels < vel_low and density = dens_high for vels > vel_high).
! If specify_dens_coeff = .false, then a default equation is used,
! based on Boore (BSSA, 2016, pp. 316--320).

! Suggested values if specify coefficients (but using the builtin function
! is preferred, as is done if specify_dens_coeff = .false) (WARNING: these have NOT been 
! modified with consideration of the results in Boore, 2016):

!        dens_low = 2.0
!        vel_low = 0.1
!        dens_high = 2.7
!        vel_high = 5.0
!        dd_dv = 0.0

 
! Another relation to consider  density = 1.73 + 0.29* vels  
! (From fitting data in a model provided by H. Magistrale, probably based
! on a Nafe & Drake relation). (NOTE added 12 october 2010:  I found the data and a plot showing the
! regression equation.  It was dens = 1.742 + 0.2875*Vs; I redid the regression today
! and confirmed the values.  I do not know where I obtained the 1.73 value mentioned above).

! Dates: 12/05/95 - Written by D.M. Boore
!        12/08/95 - Added specify_dens_coeff and the coefficients
!        04/16/96 - Changed vel_low from 0.291 to 0.3
!        02/10/03 - changed slope_v to dd_dv and included it in parameter
!                   list (it was not saved before, which was an error).
!        03/01/07 - Use a different default relation between S-wave
!                   velocity and density (see my notes on the subject).
!                   Added character variable "units_c": kps, mps for 
!                   km/sec or m/sec.  The development of this relation is
!                   described in daves_notes_on_relating_density_to_velocity_v1.2.pdf,
!                   available from http://quake.wr.usgs.gov/~boore/daves_notes.php.
!        04/29/15 - Replaced comment characters * or C with ! (The Fortran 95 standard)
!        07/09/15 - Replace default relation with new one from daves_notes_on_relating_density_to_velocity_v2.0.pdf
!        11/08/15 - Replace default relation with new one from daves_notes_on_relating_density_to_velocity_v3.0.pdf
!        07/09/18 - The only change was that the default relation has now been published as 
!                   Boore (BSSA, 2016, pp. 316--320).  The equations are the same as for the 11/08/15 revision.
!        02/22/22 - Changed "(vel. ge. 0.30 .and. vel. lt. 3.55)" to "(vel .ge. 0.30 .and. vel .lt. 3.55)" (The previous version
!                   worked, but I do not like the space between "." and "lt" and "ge")

      character units_c*(*), units_uc*3
      logical specify_dens_coeff, firstcall
      data firstcall/.true./

!      if (firstcall) then
!        firstcall = .false.
!        if (.not. specify_dens_coeff) then
!          dens_low = 2.5
!          vel_low = 0.3
!          dens_high = 2.8
!          vel_high = 3.5
!        end if
!        dd_dv = (dens_high - dens_low)/(vel_high - vel_low)
!      end if

      call trim_c(units_c, nc_units_c)
      units_uc = ' '
      units_uc = units_c(1:3)
      call upstr(units_uc)
      

      if (.not. specify_dens_coeff) then
        if (units_uc(1:3) .eq. 'KPS') then
           vel = vels
        else if (units_uc(1:3) .eq. 'MPS') then
           vel = vels/1000.0
        else
           print *,' ERROR, units_uc = '//units_uc(1:3)//
     :             ' which is not a valid string; QUITTING'
           stop
        end if
        
        if (vel .lt. 0.30) then
          density = 1.0 + (1.53*vel**0.85)/(0.35+1.889*vel**1.7)
!          density = 1.0 + (1.22*vel)/(0.0948+vel)
        else
          vp = 0.9409 + 2.0947*vel -0.8206*vel**2 + 0.2683*vel**3
     :                                            - 0.0251*vel**4
          if (vel .ge. 0.30 .and. vel .lt. 3.55) then
            density = 1.74 * vp**0.25
          else
            density = 1.6612*vp - 0.4721*vp**2 + 0.0671*vp**3
     :                          - 0.0043*vp**4 + 0.000106*vp**5 
          end if
        end if
        
      else
        if (vels .lt. vel_low) then
          density = dens_low
        else if (vels .gt. vel_high) then
          density = dens_high
        else
          dd_dv = (dens_high - dens_low)/(vel_high - vel_low)
          density = dens_low + (vels - vel_low)*dd_dv
        end if
      end if

      return
      end
! --------------------------- END DENSITY ----------------------------
!************************************************
      subroutine nhrpclss(avgv, class)
!**   Classify soil type based on velocity over top 30 m.
! Dates: 08/28/03 - written by D. Boore, based on subroutine class in program
!                   assgncls
!        04/29/15 - Replaced comment characters * or C with ! (The Fortran 95 standard)

      character*1 class
      if (avgv .gt. 1500.) then
        class = 'A'
      else if (avgv .le. 1500. .and. avgv .gt. 760.) then
        class = 'B'
      else if (avgv .le.  760. .and. avgv .gt. 360.) then
        class = 'C'
      else if (avgv .le.  360. .and. avgv .ge. 180.) then
        class = 'D'  ! .ge. 180 according to NEHRP document
      else                !  (avgv .lt.  180.) is implied
        class = 'E'
      end if

      return
      end 
!************************************************
! -------------------------- BEGIN EtaForSRImodFromFunction --------------------------
      function EtaForSRImodFromFunction(f, f2b)
 
 
      a=0.56
      b=log10(0.05)
      s=log10(1.25/0.05)
      d=4.0
      e=6.0
      f4eta=2.0
      g=0.76
      h=3.0
      p=1.0/3.0


      fNorm = f/f2b
      x = log10(fNorm)

      EtaForSRImodFromFunction = 
     :   a*((x-b)/s)**d/((1-((x-b)/s)**e)**f4eta + g*((x-b)/s)**h)**p

      return
      end
! -------------------------- END EtaForSRImodFromFunction --------------------------
      
 

 
! -------------------------- BEGIN EtaForSRImodFromCSV --------------------------
      function EtaForSRImodFromCSV(f, f2b, alogfFromEtaCSV, 
     :                             meanEtaFromEtaCSV,nEtaCSV)
 
      real alogfFromEtaCSV(*), meanEtaFromEtaCSV(*)

      dlogfNorm = 0.00291
      
      fNorm = f/f2b
      x = log10(fNorm)

!DEBUG
!      write(*,*) ' In EtaForSRImodFromCSV: ', f, f2b
!DEBUG
      
      call locate(alogfFromEtaCSV,nEtaCSV,x,j)
      if (j==0) then 
        EtaForSRImodFromCSV = meanEtaFromEtaCSV(1)
      else if (j==nEtaCSV) then
        EtaForSRImodFromCSV = meanEtaFromEtaCSV(nEtaCSV)
      else
        eps = (x-alogfFromEtaCSV(j))/dlogfNorm
        EtaForSRImodFromCSV = eps*meanEtaFromEtaCSV(j) + 
     :                 (1.0-eps)* meanEtaFromEtaCSV(j+1)
      end if

! eps = (x-xx(j))/dlogfNorm

! interp = eps*y(j) + (1-eps)*y(j+1)

! Comments added by D. Boore on 26feb2010:
!  finds j such that xx(j) < x <= xx(j+1)
!  EXCEPT if x = xx(1), then j = 1 (logically it would be 0 from
!  the above relation, but this is the same returned value of j
!  for a point out of range).
!  Also, if x = xx(n), j = n-1, which is OK
!  Note that j = 0 or j = n indicates that x is out of range.
!
!      EtaForSRImodFromFunction = 
!     :   a*((x-b)/s)**d/((1-((x-b)/s)**e)**f4eta + g*((x-b)/s)**h)**p

      return
      end
! -------------------------- END EtaForSRImodFromCSV --------------------------
      
 

  
      subroutine construct_constant_velocity_model(
     : ndepthsOriginalModel,
     : depth, vel, dens, qi,  
     : vel_ref, dens_ref, aoi_src, p,
     : akappa_fixed, 
     : freqlow, freqhigh, nfreq_in, log_spaced_freqs, 
     : d2lyrbot, thick_layer, vel_layer, dens_layer, 
     : qi_kappa_fixed, qi_layer, 
     : tt2lyrbot, avgvel2lyrbot, avgdens2lyrbot, avgaoi2lyrbot,
     : akappa2lyrbot,
     : freq_out, amp_out, akappa_out, 
     : ampxkappa_out, akappa4amp, nout,
     : fmin4OrigModel, MLAYR, MNFREQS )

!Input variables: 
!      ndepthsOriginalModel,  
!      depth, vel, dens, qi, 
!      vel_ref, dens_ref, aoi_src, p,
!      akappa_fixed, 
!      freqlow, freqhigh, nfreq_in, log_spaced_freqs, 

!Output variables:
!      d2lyrbot, thick_layer, vel_layer, dens_layer, 
!      qi_kappa_fixed, qi_layer, 
!      tt2lyrbot, avgvel2lyrbot, avgdens2lyrbot, avgaoi2lyrbot,
!      akappa2lyrbot,
!      freq_out, amp_out, akappa_out, ampxkappa_out, nout,
!      fmin4OrigModel  !nout is the number of CVL layers

! Dates: 07/25/17 - Written by D. Boore
!        07/28/17 - Apportion kappa_fixed only down to bottom of the input model
!        07/29/17 - Modify qi model for input with gradients (the proper way is to constrain integral qi/vel, both 
!                    qi and vel having linear depth dependence; I am not doing that here (I need an analytic integral).
!                 - Note added 03/15/22: I now use the correct integral
!        08/01 17 - The meaning of slope(i) has changed.  It is now the slope between i-1 and i, not between i and i+1.
!        08/02/17 - Check for repeated depths in finding j using locate.
!        08/04/17 - Fix error in computing avgdens below the bottom of the model.
!        05/08/21 - Write more info in debug file
!        05/09/21 - Fix bug found by Chuanbin Zhu that the a*k column was different for different nfreqs.  The bug was
!                   that akappa did not include attenuation from the nfreq-1
!                   layer to the halfspace depth.  I will redo the computation of kappa in a later version, for which I will
!                   specify the depth above which kappa will be computed and if in the halfspace, the qinverse to be used in the halfspace.
!                   I will also delete the kappa computation in the loop over output layers and instead will compute kappa from the input model.
!                   But for now I will compute kappa from the i-1 layer to the halfspace, with no attenuation below the halfspace (in
!                   keeping with the previous assumption).
!        08/27/21 - Add some debug statements (to construct_constant_velocity_model.for.2021-05-09b version)
!        08/28/21 - The calculation of the kappa to be used in the exp(-pi*kappa*f) overall operator was a mess, so I have removed 
!                   that computation of kappa. For possible future use I have simply commented out the kappa computation code.  I still apply the overall
!                   kappa obtained from site_amp_batch.ctl (akappa_fixed, from the line below "! kappa for computation (enter 0.0 for no kappa computation):"). 
!                   If a model contains Q in layers, site_amp_batch will compute the overall kappa from the surface to the bottom of the velocity model 
!                   and will combine this with akkapa_fixed in the overall kappa operator. A layered Q model can also (and probably more appropriately) be
!                   used in nrattle computations (see discussion in Boore, 2013).
!        03/15/22 - Reinstate the computation of kappa, and qinverse in the equivalent layered model, so that f4nrattle will create a nrattle control file
!                   that contains qinverse.
!                 - Attempt to simplify code when possible.  For example, there is no reason to treat constant and linear density and also 
!                   constant and linear qinverse separately; just use the linear equations with slope=0 if constant.
!                 - Remove power_slowness_ratio and associated code.
!        02/07/24 - Added option to replace freqlow with fmin4model
!        02/08/24 - freqlow was being replaced with fmin4model when nfreqs < 0.  This caused a bug when running nRattleBatch for a bunch of velocity models,
!                   all of which should have been processed with the same freqlow. It's OK for the output freqlow to be the actual one used (and changed
!                   from the input freqlow), but nfreq cannot be changed to abs(nfreq). See the code below for the fix.
!        02/09/24 - Changed "loop over output layers" to "loop over frequencies", as this better describes what is happening.  For each sorted
!                   frequency (in decreasing order), a new CVL is added to the stack of layers (starting from the top layer and working down).
!        02/10/24 - Major restructuring, putting the code in the main program after a piecewise continuous model has been developed into this 
!                   construct_constant_velocity_model subroutine.  Doing this allows
!                   for extrapolating the input model, if desired, within the main program, and then calling construct_constant_velocity_model
!                   with that model.  I do need to compute and save fmin4OrigModel and fmin4model_final, as those frequencies will be used in constructing
!                   normalized frequencies for use in a later program that computes FR, SRI, and eta in one program.
!        02/23/24 - Removed ndepthsAfterAnyExtrapolation and fmin4XtrapModel variable. It was there in case an extrapolation option of the original model was an option, but
!                   I decided against including that option.

      implicit none
      
      integer MNFREQS, nout, nfreq_in, MLAYR, ndepths_in ,nxtrap, iz,
     :        nfreq, i, j, idepth, k, ndepths, id,
     :        ndepthsOriginalModel, nu_debug

      real fmin4OrigModel, akappa4amp, freqhigh, freqlow,
     :  pi, d2r, delta_tt, delta_z, densint2lyrbot,
     :  v_i, akappa_fixed, p, aoi_src, dens_ref, vel_ref

      logical log_spaced_freqs 
      
      real depth(*), vel(*), dens(*), qi(*)
      
      real freq_out(*), amp_out(*), akappa_out(*), ampxkappa_out(*) 
      
      real slope_v(MLAYR), slope_r(MLAYR), slope_qi(MLAYR)
      
      real avgvel(MLAYR), del_z(MLAYR)
      real avgdens(MLAYR),avgaoi(MLAYR),akappa(MLAYR),dens_int(MLAYR)
      
      real thick_layer(*), vel_layer(*), dens_layer(*),
     :     qi_kappa_fixed(*), qi_layer(*), 
     :     d2lyrbot(0:*), tt2lyrbot(0:*), 
     :     avgdens2lyrbot(0:*), akappa2lyrbot(0:*), 
     :     avgvel2lyrbot(*), avgaoi2lyrbot(*), tt(MLAYR) 

      integer iout(MNFREQS)
  
!!DEBUG
!        call get_lun(nu_debug)
!        open(unit=nu_debug,file='debug_in_subroutine.out',
!     :         status='unknown')
!        
!        write(nu_debug, *)
!        write(nu_debug, '(a,1x,i5)') ' nfreq_in = ', nfreq_in
!        write(nu_debug, *)
!!DEBUG

      pi = 4.0 * atan(1.0)
      d2r = pi/180.0

! COMPUTE AVERAGES FROM THE SURFACE TO THE BOTTOM OF EACH MODEL LAYER.  THIS INFORMATION IS USED TO
! COMPUTE THE AVERAGES NEEDED FOR THE SITE AMPLIFICATIONS AND THE EQUIVALENT CONSTANT VELOCITY MODEL
! FOR THE SPECIFIED FREQUENCIES IN THE OUTPUT MODEL.
        
        slope_v = 0.0
        slope_r = 0.0
        slope_qi = 0.0
        tt = 0.0
        avgvel = 0.0
        avgdens = 0.0
        avgaoi = 0.0
        akappa = 0.0
        del_z = 0.0
        
        avgvel(1) = vel(1)
        avgdens(1) = dens(1)
        tt = 0.0
        akappa = 0.0
        dens_int = 0.0

! The meaning of the quantities below can be confusing.  Here is their meaning:
! del_z(i) is the depth increment between layer i-1 and layer i.
! tt(i), avgdens(i), akappa(i) are the travel times and average values from the surface to the bottom of layer i
! slope_v(i), slope_r(i), and slope_qi(i) are the slopes from layer i-1 to layer i

!COMPUTE TT, AVGVEL, AVGDENS, AVGAOI FROM THE SURFACE TO THE BOTTOM OF EACH LAYER IN THE INPUT MODEL (After possible extrapolation):

        ndepths = ndepthsOriginalModel

        DO id = 2, ndepths
          if(depth(id) == depth(id-1)) then  ! repeated depth, usually indicates a step change in parameters
            slope_v(id) = 1.0e10
            tt(id) = tt(id-1)
            slope_r(id) = 1.0e10
            dens_int(id) = dens_int(id-1)
            slope_qi(id) = 1.0e10
            akappa(id) = akappa(id-1)
          else
            del_z(id) = depth(id)-depth(id-1)
            slope_r(id) = (dens(id)-dens(id-1))/del_z(id)
            dens_int(id) = dens_int(id-1) + dens(id-1)*del_z(id)+ 
     :          0.5 * slope_r(id) * del_z(id)**2	      
            slope_qi(id) = (qi(id)-qi(id-1))/del_z(id)
            if (vel(id) == vel(id-1)) then   ! constant velocity
              slope_v(id) = 0.0 
              tt(id) = tt(id-1) + del_z(id)/vel(id)
              akappa(id) = akappa(id-1)+ del_z(id) * qi(id-1)/vel(id) +
     :                     0.5 * (slope_qi(id)/vel(id))*del_z(id)**2
            else
              slope_v(id) = (vel(id)-vel(id-1))/del_z(id)
              tt(id) = tt(id-1) + 
     :               (1.0/slope_v(id))*alog(vel(id)/vel(id-1))
              akappa(id) = akappa(id-1)+ 
     :           del_z(id) * slope_qi(id)/slope_v(id)+
     :          (1.0/slope_v(id)**2)*
     :          (qi(id-1)*slope_v(id)-vel(id-1)*slope_qi(id))*
     :          alog(vel(id)/vel(id-1))
            end if
          end if
          avgvel(id) = depth(id)/tt(id)
          avgdens(id) = dens_int(id)/depth(id)
          avgaoi(id) = acos(sqrt(1.0-(p*avgvel(id))**2))/d2r
        END DO

!! DEBUG
!        write(nu_debug,*)
!        write(nu_debug,'(1x,a, 3(1x,i6))') 
!     : 'ndepths, ndepthsOriginalModel: ',  
!     :  ndepths, ndepthsOriginalModel   
!        write(nu_debug,*)
!! DEBUG


        fmin4OrigModel = 1.0/(4.0*tt(ndepthsOriginalModel))  ! ndepths is the number of depths in the input velocity model

!! DEBUG
!        write(nu_debug,*)
!        write(nu_debug,'(1x,a, 2(1x,es10.3))') 
!     :    'fmin4OrigModel: ',  
!     :     fmin4OrigModel   
!        write(nu_debug,*)
!! DEBUG

!! DEBUG
!        write(nu_debug,*)
!        write(nu_debug,'(1x,a, 1x,i6)') 'ndepths_model = ', ndepths
!        write(nu_debug,*)
!        write(nu_debug,'(4x,a, 6x,a, 6x,a, 8x,a, 4x,a, 7x,a, 4x,a,
!     :  9x,a, 3x,a, 9x,a, 3x,a, 4x,a, 
!     :  5x,a, 5x,a, 5x,a)') 
!     :  'id', 'depth', 'del_z', 'vel', 'slope_v', 'dens', 'slope_r',
!     :  'qi', 'slope_qi', 'tt', 'dens_int', 'avgdens', 
!     :  'avgvel', 'avgaoi', 'akappa'
!! DEBUG

! Now add an interface above the halfspace:
        slope_v(ndepths+1) = 1.0e10
        slope_r(ndepths+1) = 1.0e10
        slope_qi(ndepths+1) = 1.0e10
        vel(ndepths+1) = vel_ref
        dens(ndepths+1) = dens_ref
!        qi(ndepths+1) = qi(ndepths)             
        qi(ndepths+1) = 0.0   ! 03/17/22             
        tt(ndepths+1) = tt(ndepths)
        avgvel(ndepths+1) = avgvel(ndepths)
        avgdens(ndepths+1) = avgdens(ndepths)
        akappa(ndepths+1) = akappa(ndepths)
        depth(ndepths+1) = depth(ndepths)
        dens_int(ndepths+1) = dens_int(ndepths)
        ndepths = ndepths + 1

!!DEBUG
!        DO id = 1, ndepths
!          write(nu_debug,'(1x,i5, 15(1x,es10.3))')  
!     :     id, depth(id), del_z(id),
!     :     vel(id), slope_v(id), 
!     :     dens(id), slope_r(id),
!     :     qi(id), slope_qi(id), 
!     :     tt(id), dens_int(id), avgdens(id), 
!     :     avgvel(id), avgaoi(id), akappa(id)
!        END DO
!!DEBUG
      tt2lyrbot(0) = 0.0
      d2lyrbot(0) = 0.0
      avgdens2lyrbot(0) = 0.0
      akappa2lyrbot(0) = 0.0
        
      ndepths_in = ndepths

      IF (nfreq_in /= 0) THEN   ! SPECIFY FREQUENCIES; THIS IS A LONG SECTION

        nfreq = abs(nfreq_in)
        if (nfreq_in < 0) freqlow = fmin4OrigModel
 
        call fill_freq_array(freq_out, nfreq, freqlow, freqhigh, 
     :                        log_spaced_freqs)                      !sorted in decreasing order
      
!!DEBUG
!!
!      write(nu_debug,'(1x,a, 1x,i4, 2(1x,es10.3), 1x,l1)')
!     : 'nfreq, freqlow, freqhigh, log_spaced_freqs',
!     :  nfreq, freqlow, freqhigh, log_spaced_freqs
!!
!!DEBUG


!!DEBUG
!!
!        print *, '(nfreq = ', nfreq
!        write(nu_debug,'(1x,a, 1x,i5, 1x,a, 1x,i5)') 
!     :    ' ndepths = ', ndepths, ' nout = ', nfreq    !nuber of CVL layers is the same as nfreq
!     
!        write(nu_debug,*)
!        
!        write(nu_debug,'(4x,a, 
!     :                   6x,a, 8x,a, 7x,a, 9x,a,
!     :                   9x,a, 
!     :                   4x,a, 4x,a, 3x,a, 
!     :                   5x,a)') 
!     :   'j',   
!     :   'depth', 'vel', 'dens', 'qi', 
!     :   'tt',
!     :   'slope_v', 'slope_r', 'slope_qi', 
!     :   'akappa'
!        do j = 1, ndepths
!          write(nu_debug,'(1x,i4, 20(1x,es10.3))') 
!     :    j,  
!     :    depth(j), vel(j), dens(j), qi(j),
!     :    tt(j), 
!     :    slope_v(j), slope_r(j), slope_qi(j),
!     :    akappa(j)
!        end do
!        
!        write(nu_debug,*)
!        
!        write(nu_debug,'(4x,a, 1x,a, 4x,a, 6x,a,
!     :                   3x,a, 5x,a, 4x,a, 6x,a,
!     :                   1x,a, 1x,a, 1x,a,
!     :                   2x,a, 4x,a, 1x,a, 1x,a,
!     :                   1x,a,   
!     :                   1x,a, 1x,a
!     :                                     )') 
!     :   'i', 'tt2lyrbot(i)', 'j', 'tt(j)', 
!     :   'depth(j)', 'vel(j)', 'dens(j)', 'qi(j)', 
!     :   'slope_v(j)', 'slope_r(j)', 'slope_qi(j)',  
!     :   'akappa(j)', 'delta_z', 'd2lyrbot(i)', 'thick_layer(i)',  
!     :   'vel_layer(i)',  
!     :   'avgdens2lyrbot(i)', 'dens_layer(i)' 
!!
!!DEBUG

! Loop over frequencies:

        nout = nfreq  !nout is the number of CVL layers


        loop over frequencies: DO i = 1, nout    ! freq must be in descending order; this is done in fill_freq_array
                                                 ! a stack of CVL layers is derived for the sorted set of frequencies.


! Compute constant layer thickness and  velocity, as well as
! avgvel2lyrbot, avgaoi2lyrbot for specified frequencies:

! Compute tt corresponding to freq_out:
          tt2lyrbot(i) = 1.0/(4.0*freq_out(i))

! Find location of layer above the one in which tt2lyrbot occurs:
  ! Comments in locate.for added by D. Boore on 26feb2010:
  !  finds j such that xx(j) < x <= xx(j+1)
  !  EXCEPT if x = xx(1), then j = 1 (logically it would be 0 from
  !  the above relation, but this is the same returned value of j
  !  for a point out of range).
  !  Also, if x = xx(n), j = n-1, which is OK
  !  Note that j = 0 or j = n indicates that x is out of range.
          call locate(tt,ndepths,tt2lyrbot(i),j)
          
          if (j /= ndepths .and. depth(j+1) == depth(j)) j = j+1  ! trap for repeated depth, not at the top of the halfspace, and adjust j

! Compute the depth corresponding to tt2lyrbot:
          delta_tt = tt2lyrbot(i) - tt(j)
          IF (j == ndepths) THEN            ! into the halfspace
            delta_z = vel_ref * delta_tt
          ELSE IF (slope_v(j+1) == 0.0) THEN        ! constant velocity
            delta_z = vel(j) * delta_tt
          ELSE
! The velocity in the layer from j to j+1 
! is slope_v(j+1), as of the July 2017 revisions to site_amp_batch          
            delta_z = vel(j)*(exp(slope_v(j+1)*delta_tt)-1.0)/
     :                slope_v(j+1)                                 
          END IF

          d2lyrbot(i) = depth(j) + delta_z
          thick_layer(i) = d2lyrbot(i) - d2lyrbot(i-1)
          vel_layer(i)  = thick_layer(i)/(tt2lyrbot(i) - tt2lyrbot(i-1)) 

! Compute average velocity, aoi:

          avgvel2lyrbot(i) = d2lyrbot(i)/tt2lyrbot(i)

!DEBUG
!      write(nu_debug,'(1x,a,1x,i4,4(1x,es10.3))') 
!     : 'i, freq_out(i), d2lyrbot(i), tt2lyrbot(i), avgvel2lyrbot(i)',  
!     :  i, freq_out(i), d2lyrbot(i), tt2lyrbot(i), avgvel2lyrbot(i)
!DEBUG

          avgaoi2lyrbot(i) = 
     :       acos(sqrt(1.0-(p*avgvel2lyrbot(i))**2))/d2r
          
! Compute the average density and kappa

!BE CAREFUL:

! averages and distances are from the top of the model layer in which the new depth exists, 
! not the previous new layer boundary.   This means that I can calculate the increment of akappa
! and avgdens using the input values for the model.

          IF (j == ndepths) THEN            ! into the halfspace, assumes constant density, qi=0.0
            densint2lyrbot = dens_int(j) + delta_z * dens_ref
            akappa2lyrbot(i) = akappa(j)  ! same kappa for all depths below the model bottom
!!DEBUG
!!
!       write(nu_debug,*) ' i=',i, ' j=',j, 
!     :   'dens_int(j)=',dens_int(j), 'delta_z=',delta_z, 
!     :   'dens_ref=',dens_ref,
!     :   'densint2lyrbot=',densint2lyrbot, 
!     :   'akappa2lyrbot(i)=',akappa2lyrbot(i),
!     :   'd2lyrbot(i)=',d2lyrbot(i)
!!
!!DEBUG
          ELSE  ! above the halfspace
            densint2lyrbot = dens_int(j) +  
     :                       dens(j)*delta_z +
     :                       0.5 * slope_r(j+1) * delta_z**2      
            IF (slope_v(j+1) == 0.0) THEN                               ! constant velocity                            ! 
              akappa2lyrbot(i) = akappa(j)+ delta_z * qi(j)/vel(j) +
     :                     0.5 * (slope_qi(j+1)/vel(j))*delta_z**2
            ELSE                                                        ! linear gradient
              v_i = vel(j) + slope_v(j+1)*delta_z
              akappa2lyrbot(i) = akappa(j)+ 
     :            delta_z * slope_qi(j+1)/slope_v(j+1)+
     :           (1.0/slope_v(j+1)**2)*
     :           (qi(j)*slope_v(j+1)-vel(j)*slope_qi(j+1))*
     :           alog(v_i/vel(j))
            END IF
          END IF

          avgdens2lyrbot(i) = densint2lyrbot/d2lyrbot(i)
            
            
 ! Compute average akappa, qi_layer, dens_layer:

          amp_out(i) = sqrt( (dens_ref*vel_ref) /
     :     (avgdens2lyrbot(i)*avgvel2lyrbot(i)) )
          amp_out(i) = amp_out(i) * 
     :       sqrt(cos(d2r*aoi_src)/cos(d2r*avgaoi2lyrbot(i))) ! include cos fctr
          qi_layer(i) = ( akappa2lyrbot(i) - akappa2lyrbot(i-1) ) /
     :                    ( tt2lyrbot(i) - tt2lyrbot(i-1) )
          dens_layer(i)  = ( d2lyrbot(i)  * avgdens2lyrbot(i) - 
     :                      d2lyrbot(i-1)* avgdens2lyrbot(i-1) )/
     :                      thick_layer(i)

!!DEBUG
!!
!          write(nu_debug,'(1x,i4, 3x,es10.3, 1x,i4, 1x,es10.3,
!     :                     1x,es10.3, 1x,es10.3, 1x,es10.3, 1x,es10.3,
!     :                     1x,es10.3, 1x,es10.3, 2x,es10.3,
!     :                     1x,es10.3, 1x,es10.3, 2x,es10.3, 5x,es10.3,
!     :                     3x,es10.3, 
!     :                     8x,es10.3, 4x,es10.3)' )     
!     :    i, tt2lyrbot(i), j, tt(j), 
!     :    depth(j), vel(j), dens(j), qi(j),
!     :    slope_v(j), slope_r(j), slope_qi(j),
!     :    akappa(j), delta_z, d2lyrbot(i), thick_layer(i), 
!     :    vel_layer(i),  
!     :    avgdens2lyrbot(i), dens_layer(i)
!!     
!DEBUG

        END DO loop over frequencies

      ELSE   ! FREQUENCIES NOT SPECIFIED; RETURN CVL MODEL AT DEPTHS IN THE INPUT MODEL

! Fill index array for output so that will not repeat information at the
! same depth
        nout = 0
        DO idepth = 2, ndepths
          if (depth(idepth) /= depth(idepth-1)) then ! only count if not repeated depth
!          if (depth(idepth) == depth(idepth-1)) goto 777 ! skip repeated depths [previous version]
            nout = nout + 1
            iout(nout) = idepth
          end if
!777       continue
        END DO

        loop over depths: DO i = 1, nout                       ! note that nout <= ndepths

          k = iout(i)
          thick_layer(i) = depth(k) - depth(k-1)
          vel_layer(i)   = thick_layer(i)/(tt(k) - tt(k-1)) 
          dens_layer(i)  = ( depth(k)   * avgdens(k) - 
     :                    depth(k-1) * avgdens(k-1) )/
     :                    thick_layer(i)
          freq_out(i)    = 1.0/(4.0*tt(k))
          
          avgvel(k) = depth(k)/tt(k)
          amp_out(i)     = sqrt( (dens_ref*vel_ref) /
     :                        (avgdens(k)*avgvel(k)) )
          amp_out(i)     = amp_out(i) * 
     :           sqrt(cos(d2r*aoi_src)/cos(d2r*avgaoi(k))) ! include cos fctr
          qi_layer(i)    = ( akappa(k) - akappa(k-1) )/
     :                  ( tt(k)  -    tt(k-1) )

          d2lyrbot(i) = depth(k)            
          tt2lyrbot(i) = tt(k)            
          akappa2lyrbot(i) = akappa(k)            
          avgvel2lyrbot(i) = avgvel(k)
          avgdens2lyrbot(i) = avgdens(k)
          avgaoi2lyrbot(i) = avgaoi(k)

        END DO loop over depths
        
      END IF            ! if over specified freq or not
!

 
!Fill some of the other output arrays:

!!DEBUG
!      write(nu_debug,*)
!      write(nu_debug,'(1x,a)') 
!     :  ' i, nout, akappa2lyrbot, akappa_fixed, akappa_out'
!DEBUG
      do i = 1, nout      
        akappa_out(i) = akappa2lyrbot(i) + akappa_fixed
!DEBUG
!      write(nu_debug,*) 
!     :  i, nout, akappa2lyrbot(i), akappa_fixed, akappa_out(i)
!DEBUG
      end do
      
      akappa4amp = akappa(ndepths-1) + akappa_fixed

!!DEBUG
!      write(nu_debug,*)
!      write(nu_debug,'(1x,a, 1x,i5, 3(1x,es10.3))') 
!     :  ' ndepths, akappa(ndepths-1), akappa_fixed, akappa4amp = ',
!     :    ndepths, akappa(ndepths-1), akappa_fixed, akappa4amp
!DEBUG
!!Note that the previous version checked for q_from_model, but this is done in the driver
!!and qi_layer is set to 0.0 if .false.

      do i = 1, nout
!!DEBUG
!         write(*, '(1x,a, 1x,i4, 
!     :                    1x,a, 1x,es10.3,
!     :                    1x,a, 1x,f5.3, 
!     :                    1x,a, 1x,es10.3, 
!     :                    1x,a, 1x,es10.3)') 
!     :  'i',i, 
!     :  'amp_out',amp_out(i),
!     :  'pi',pi,
!     :  'akappa4amp',akappa4amp,
!     :  'freq_out',freq_out(i)
!!DEBUG
     
        ampxkappa_out(i) = amp_out(i)*exp(-pi*akappa4amp*freq_out(i))
      end do

!DEBUG
!       close(nu_debug)
!DEBUG
      
      
      return
      end
      
      
!      !@@@@@@@
!               IF (.not. q_from_model) akappa(k) = 0.0
!               IF (.not. q_from_model) akappa(ndepths) = 0.0
!               akappa4amp = akappa(ndepths) ! 
!           :                + akappa_fixed
!           
!               write(cbuf(16), '(es11.5)') akappa4amp
!               write(cbuf(17), '(es11.5)') 
!           :     amp_out(i)*exp(-pi*akappa4amp*freq) 
!               write(cbuf(18), '(es11.5)') 
!           :    fmin4model
!           
                
! --------------------- BEGIN Fill_Freq_Array ------------------------------
      subroutine fill_freq_array(freq, nfreq,
     :            freqlow, freqhigh, log_spaced_freqs)
      real freq(*)
      logical log_spaced_freqs
      
      do i = 1, nfreq
        if (log_spaced_freqs) then
          freq(i) = 
     :      freqlow * (freqhigh/freqlow)**(real(i-1)/
     :       real(nfreq-1))
        else
          freq(i) = freqlow + real(i-1)*(freqhigh - freqlow)/
     :       real(nfreq-1)
        end if
      end do
 
! Sort frequencies and reorder in decreasing order (so output will have
! depth increasing):

      call sort(nfreq, freq)

      nswap = nfreq/2
      do i = 1, nswap
        temp = freq(nfreq-i+1)
        freq(nfreq-i+1) = freq(i)
        freq(i) = temp
      end do
        
      return
      end
! --------------------- END Fill_Freq_Array ------------------------------
 
      SUBROUTINE NrattleAmpComputations(z, h, vs, rho, qis, 
     :          nlay, nlay_src, betan, rhon, theta, zdepth, 
     :          freq, nfreq, CmplxAmp, mlayr, mnfreqs)

      implicit none

      real Z(*), H(*), vs(*), rho(*), qis(*), freq(*)
      complex CI,CY,CZ,VV,
     :        XMU(MLAYR),XXX(MLAYR), CmplxAmp(MNFREQS)
!-----
!
!	Z	Real	Depth to bottom of layer from surface
!	H	Real	Layer thickness
!	XMU	Complex	Rigidity of layer
!	XXX	Complex	Vertical slowness of layer for  given 
!			angle of incidence
!	A	Complex	Wave propagation coefficients for the layer
!	B	Complex
!	EXE	exponent for extended floating point
!-----
      complex Q11,Q12,Q21,Q22
      complex A(MLAYR), B(MLAYR), EXE(MLAYR)
      complex facz
      
      integer nfreq, nlay_src, nlay, mlayr, mnfreqs, n, i, IFRQ, j, ii
      
      real ds, dexpp, zdepth, theta, rhon, betan, hslow, hs2, tstar, 
     :     f, w, facr, faci, expp, expm, pi
     
! On to the computations:

      CI = (0.0,1.0d+00)
      PI = 4.0*ATAN(1.0)


        if (nlay_src > nlay) then
          n = nlay + 1
        else
          n = nlay_src
          nlay = nlay_src - 1
! Use layer values of the effective half space is less than the real half space
          betan = vs(n)
          rhon = rho(n)
        end if

        theta = theta*pi/180.0
        hslow = sin(theta)/betan
        hs2 = hslow**2

        xmu(n) = rhon*betan*betan
        xxx(n) = csqrt((rhon/xmu(n)-hs2)*cmplx(1.0d+00,0.0d+00))

!-----RBH
!	note: Mueller had XMU(I) = RHO*VS*VS*(1.0d+00-CI/QS)
!	The -CI is changed to CI because of the Fourier Transform convention
!	that Herrmann uses, AND, a more exact expression for the velocity
!       squared is used, e.g.,
!       (1.0d+00+0.5*CI/QS) ~~ (1.0d+00-CI/QS) if 1/QS << 1
!-----RBH

!       and note that now 1/Q is used as the measure of attenuation,
!       so what was CI/QS in the old version of the program and in the
!       comments above is now CI*QiS.

        do i = 1, nlay
         xmu(i) = rho(i)*vs(i)*vs(i)*
     :            (1.0d+00+0.5*ci*qis(i))*(1.0d+00+0.5*ci*qis(i))
         xxx(i) = csqrt(rho(i)/xmu(i)-hs2)
        end do

!	sumtq = 0.0
!	sumt = 0.0
!	WRITE(nu_db,90261)
!90261 format(
!     1 ' LYR           XMU(i)         ',
!     2 '        D(i)     Z(i)   sum(t/q)    sum(t)        VS',
!     3 '      QiS')
!      DO I=1,N  ! note: array values undefined if n > nlay
!	sumtq = sumtq - 2.0*aimag(xxx(i))*h(i)
!	sumt = sumt + real(xxx(i))*h(i)
!         WRITE (nu_db,90260) I,XMU(I),H(I),Z(I),sumtq,sumt, vs(I),
!     :                   qis(I)
!90260    FORMAT ('(',i3,')=',2E12.4,'  ',5f10.3,1x,1pe10.3)
!      end do

!-----RBH
!	compute the surface motion for a basal input.
!	This is described in S. Kramer's, Geotechnical Earthquake Engineering
!
!	THEORY:
!	Define a local coordinate system, with Z down, X to the right.
!	The surface is at Z = 0
!	The SH displacement field in a constant velocity layer 'i' is
!
!	Ui(z) = Ai exp (nu z ) + Bi (- nu z)
!
!	where
!
!	nu = i omega sqrt( 1/VS**2 - 1/p**2 )  (p = ray parameter)
!	       omega sqrt(-1/VS**2 + 1/p**2 )  (for evanescent wave)
!
!	with this convention, the coefficient Ai is for a plane wave
!	propagation in the +x, -z direction with increasing time, e.g.,
!	upward, and the Bi coefficient is for downward propagation
!
!	At the top of layer i,
!	     Ui = Ai + Bi
!	At the base of layer i, which has thickness Hi
!	     Ui = Ai exp (nu Hi ) + Bi (- nu Hi)
!	At the top of layer i+1,
!	     Ui+1 = Ai+1 + Bi+1
!	Continuity of displacement and stress across the layer boundary
!	permits a matrix relation between the A and B coefficients
!
!	| A |     | (1/2)(1+r) exp (nui hi) (1/2)(1-r) exp(- nui hi)||A|
!       |   |  =  |                                                 || |
!	| B |     | (1/2)(1-r) exp (nui hi) (1/2)(1+r) exp(- nui hi)||B|
!       |   |i+1  |                                                 || |i
!
!	where r is not a reflection coefficient, but the ratio
!		r = mu(i) nu(i)/(mu(i+1) nu(i+1)
!	where the complex rigidity is rho beta**2 (1 + i/2Q)**2
!
!	The free surface condition is met by A1 = B1.
!	Thus if we propagate this downward, we have can compute A(N) which
!	is the value of the upward, incident plane wave field
!	as a function of A1, the upward at the surface.
!	
!	The plane wave response at the surface for an incident wavefield is
!		A1 + B1 = 2 A1
!	If we wish to compute ONLY the amplification with respect to a
!	location with NO surface layering, e.g., a hard rock reference site,
!	then the relative site amplification is just
!
!	VV = A(1)/A(N)
!
!	Note: the parameter ZDEPTH is  used here since we do  wish
!	to permit a pseudo-VSP
!
!	CONSEQUENCES:
!	In the case of medium response with Q, we physically expect the
!	surface wavefield to be attenuated compared to the input at the
!	base. Both this formulation and Mueller's use the above matrix relation.
!	For low Q and high frequencies, the increasing exponential may dominate
!	the matrix. Here this implies that the AN >> A1 which is physically 
!       correct.
!	The Mueller expression gives A1 > AN due to roundoff error.
!
!	Neither expression uses Kennett's recursive approach, which starts
!	of course with the above matrix equation, but which will not
!	have any increasing exponential term, and is thus guaranteed stable
!
!	FINALLY: the code presented here will blow up nuerically in single
!	precision at high frequencies because of the finite size of the exponent
!	One should actually use the same care the Herrmann uses in hspec96
!-----RBH

!-----
!	Numerical note
!	To avoid problems at high frequencies with increasing
!	exponentials, we will compute an A' and B'
!	which are related to A and B by A = A' exp(fac), B=B' exp(fac)
!**********
!
!  LOOP OVER FREQUENCY
!
!**********

	tstar = 0.0

        loop over frequencies: DO IFRQ=1,nfreq

          f = freq(ifrq)
          w = 2*pi*f
!**********
!
!  LOOP OVER THE NUMBER OF LAYERS (PROPAGATE).
!
!**********
	  a(1) = cmplx(1.0d+00,0.0d+00)
	  b(1) = cmplx(1.0d+00,0.0d+00)
	  do i=1,nlay
	    exe(i) = 0.0
	  enddo
          do i=1,nlay
            j = i+1
            cy = (xmu(i)/xmu(j))*(xxx(i)/xxx(j))
            cz = ci*w*h(i)
            facr =  real(cz*xxx(i))
            faci = aimag(cz*xxx(i))
            facz = cmplx(cos(faci), sin(faci))
            
            expp = 1.0
            expm = -facr -facr
            
            q11 =  0.5*(1.0+cy)*facz
            q21 =  0.5*(1.0-cy)*facz
            if(expm .lt. -50.0)then
		q12 = 0.0
		q22 = 0.0
	    else
            	q12 =  0.5*(1.0-cy)*conjg(facz)*exp(expm)
            	q22 =  0.5*(1.0+cy)*conjg(facz)*exp(expm)
            endif
            
	    a(j) = q11*a(i) + q12*b(i)
	    b(j) = q21*a(i) + q22*b(i)
	    exe(j) = exe(i) + facr
!                call abnorm(a(j),b(j),facn)
!		exe(j) = exe(j) + facn
	    if(ifrq == 1 .and. i <= nlay)then
	      tstar = tstar - 2.0*aimag(xxx(i))*h(i)
	    endif
          end do
!*******************

          i = n

!-----rbh
!	compute the layer stack response at the desired depth zdepth beneath 
!       the surface
!       relative to the base input for a halfspace model.
!
!	first determine the layer in which the source lies, and the distance of
!	the source from the top of that layer
!-----RBH
	  if(zdepth >= z(nlay)) then
!-----
!	desire motion in halfspace
!-----
	    ii = nlay + 1
	    ds = zdepth - z(nlay)
	    cz = ci*w*ds
	    dexpp = exe(ii) - exe(n)
	    if(dexpp < -50.0)then
	      vv = cmplx(0.0,0.0)
	    else
	      vv = (0.5*(a(ii)*cexp(cz*xxx(ii)) 
     :			+ b(ii)*cexp(-cz*xxx(ii)))/a(n))
     :			* exp( dexpp )
	    endif
!-----
	  else if (zdepth == 0.0) then
!-----
!	desire motion at the surface
!-----
	    ii = 1
	    vv = a(1) / a(n)
	    dexpp = exe(ii) - exe(n)
	    if(dexpp < -50.0)then
	      vv = cmplx(0.0,0.0)
	    else
	      vv = (a(1) / a(n)) * exp( dexpp )
	    endif
	  else
!-----
!	desire motion within a layer
!-----
	    do i = 1, nlay
  	      ds = zdepth - z(i) 
	      if(ds < 0.0) then
	        ds = -ds
	        cz = ci*w*ds
	        ii = i
	        dexpp = exe(ii) - exe(n)
	        if(dexpp < -50.0) then
	          vv = cmplx(0.0,0.0)
	        else
   	          vv = (0.5*(a(ii)*cexp(cz*xxx(ii)) 
     :			+ b(ii)*cexp(-cz*xxx(ii)))/a(n))
     :			* exp( dexpp )
                endif
                go to 1234
	      endif
	    enddo
	  endif

 1234	  continue
 
          CmplxAmp(ifrq) = vv
        
        END DO loop over frequencies  
        
      RETURN
      END
 
