! -------------------- BEGIN TT4vert -------------------------
      function tt4vert(z, d2bot, vel, nlayers)

! Dates: 03/29/99 - Written by D. Boore
!        05/09/03 - Extrapolate if z > deepest layer
!        04/29/15 - Replaced comment characters * or C with ! (The Fortran 95 standard)

      real d2bot(0:*), vel(*)

      tt4vert = 0.0
      do i = 1, nlayers
        if( z .gt. d2bot(i) ) then
          tt4vert = tt4vert + (d2bot(i)-d2bot(i-1))/vel(i)
        else
          z_in_layer = z - d2bot(i-1)  ! for this to work, need d2bot(0)=0.0
          tt4vert = tt4vert + z_in_layer/vel(i)
          go to 578
        end if
      end do
      if (z .gt. d2bot(nlayers)) then
          tt4vert = tt4vert + (z - d2bot(nlayers))/vel(nlayers)
      end if
578   continue

      return
      end
! -------------------- END TT4vert -------------------------

! --------------------------- BEGIN GET_LUN ----------------
      subroutine get_lun(lun)

! Finds a logical unit number not in use; returns
! -1 if it cannot find one.

! Dates -- 05/19/98 - Written by D. Boore, following
!                     Larry Baker's suggestion
!        04/28/15 - Replaced comment characters * or C with ! (The Fortran 95 standard)

      logical isopen
      do i = 99,10,-1
        inquire (unit=i, opened=isopen)
        if(.not.isopen) then
          lun = i
          return
        end if
      end do
      lun = -1

      return
      end
! --------------------------- END GET_LUN ----------------
     

! --------------------------- BEGIN TRIM_C -----------------------
      subroutine trim_c(cstr, nchar)

! strips leading and trailing blanks from cstr, returning the
! result in cstr, which is now nchar characters in length

! Strip off tabs also.

! Here is a sample use in constructing a column header, filled out with 
! periods:

!* Read idtag:
!        idtag = ' '
!        read(nu_in, '(1x,a)') idtag
!        call trim_c(idtag, nc_id)
!* Set up the column headings:
!        colhead = ' '
!        colhead = idtag(1:nc_id)//'......' ! nc_id + 6 > length of colhead

! Dates: 12/23/97 - written by D. Boore
!        12/08/00 - pad with trailing blanks.  Otherwise some comparisons
!                   of the trimmed character string with other strings
!                   can be in error because the original characters are left
!                   behind after shifting.  For example, here is a string
!                   before and after shifting, using the old version:
!                      col:12345
!                           MTWH  before
!                          MTWHH  after (but nc = 4).
!        03/21/01 - Check for a zero length input string
!        11/09/01 - Change check for zero length string to check for all blanks
!        10/19/09 - Strip off tabs
!        04/28/15 - Replaced comment characters * or C with ! (The Fortran 95 standard)
!        02/14/18 - Use "len_trim" to check for cstr being all blank

      character cstr*(*)

      nend = len_trim(cstr)

      if(nend == 0) then
        nchar = 0
        return
      end if

! Replace tabs with blanks:

      do i = 1, nend
        if(ichar(cstr(i:i)) == 9) then
           cstr(i:i) = ' '
        end if
      end do



!      if(nend == 0) then
!        nchar = 0
!        return
!      end if

      do i = nend, 1, -1
        if (cstr(i:i) /= ' ') then
           nchar2 = i
           goto 10
        end if
      end do

10    continue

      do j = 1, nchar2
        if (cstr(j:j) /= ' ') then
          nchar1 = j
          goto 20
        end if
      end do

20    continue
   
      nchar = nchar2 - nchar1 + 1
      cstr(1:nchar) = cstr(nchar1: nchar2)
      if (nchar < nend) then
        do i = nchar+1, nend
          cstr(i:i) = ' '
        end do
      end if

      return
      end
! --------------------------- END TRIM_C -----------------------

! --------------------- BEGIN UPSTR ----------------------------------
      Subroutine UPSTR ( text )
! Converts character string in TEXT to uppercase
! Dates: 03/12/96 - Written by Larry Baker
!        04/28/15 - Replaced comment characters * or C with ! (The Fortran 95 standard)

!
      Implicit   None
!
      Character  text*(*)
!
      Integer    j
      Character  ch
!
      Do 1000 j = 1,LEN(text)
         ch = text(j:j)
         If ( LGE(ch,'a') .and. LLE(ch,'z') ) Then
            text(j:j) = CHAR ( ICHAR(ch) - ICHAR('a') + ICHAR('A') )
         End If
 1000    Continue
!
      Return
      End
! --------------------- END UPSTR ----------------------------------

! ---------------------- BEGIN SKIP -------------------
      subroutine SKIP(lunit, nlines)
        if (nlines .lt. 1) then
          return
        else
          do i = 1, nlines
             read(lunit, *)
          end do
          return
        end if
      end
! ---------------------- END SKIP -------------------
! ------------------------------------------------------------------ skipcmnt
      subroutine skipcmnt(nu, comment, ncomments)

! Skip text comments in the file attached to unit nu, but save skipped 
! comments in character array comment.  Skip at least one line, and more as 
! long as the lines are preceded by "|" or "!".

! Dates: 04/16/01 - Written by D. Boore
!        12/07/01 - Added check for eof
!        11/04/03 - Use trim_c to trim possible leading blank
!        02/03/07 - Initialize comments to blank
!        04/28/15 - Replaced comment characters * or C with ! (The Fortran 95 standard)

      character comment(*)*(*), buf*80

      ncomments = 0
100   buf = ' '
      read (nu,'(a)',end=999) buf
      call trim_c(buf,nc_buf)
      if (buf(1:1) .eq.'!' .or. buf(1:1) .eq.'|' .or. 
     :                     ncomments + 1 .eq. 1) then
        ncomments = ncomments + 1
        comment(ncomments) = ' '
        comment(ncomments) = buf(1:nc_buf)
        goto 100
      else 
        backspace nu
      end if

999   continue
 
      return
      end
! ------------------------------------------------------------------ skipcmnt
! --------------------------- BEGIN DENSITY ----------------------------
      function density(vels,
     :                 specify_dens_coeff,
     :                 dens_low, vel_low,
     :                 dens_high, vel_high, dd_dv, units_c)

! Assigns a density based on shear velocity.  If specify_dens_coeff = .true.,
! then a straight line is fit between the coefficients (and density = dens_low
! for vels < vel_low and density = dens_high for vels > vel_high).
! If specify_dens_coeff = .false, then a default equation is used,
! based on Boore (BSSA, 2016, pp. 316--320).

! Suggested values if specify coefficients (but using the builtin function
! is preferred, as is done if specify_dens_coeff = .false) (WARNING: these have NOT been 
! modified with consideration of the results in Boore, 2016):

!        dens_low = 2.0
!        vel_low = 0.1
!        dens_high = 2.7
!        vel_high = 5.0
!        dd_dv = 0.0

 
! Another relation to consider  density = 1.73 + 0.29* vels  
! (From fitting data in a model provided by H. Magistrale, probably based
! on a Nafe & Drake relation). (NOTE added 12 october 2010:  I found the data and a plot showing the
! regression equation.  It was dens = 1.742 + 0.2875*Vs; I redid the regression today
! and confirmed the values.  I do not know where I obtained the 1.73 value mentioned above).

! Dates: 12/05/95 - Written by D.M. Boore
!        12/08/95 - Added specify_dens_coeff and the coefficients
!        04/16/96 - Changed vel_low from 0.291 to 0.3
!        02/10/03 - changed slope_v to dd_dv and included it in parameter
!                   list (it was not saved before, which was an error).
!        03/01/07 - Use a different default relation between S-wave
!                   velocity and density (see my notes on the subject).
!                   Added character variable "units_c": kps, mps for 
!                   km/sec or m/sec.  The development of this relation is
!                   described in daves_notes_on_relating_density_to_velocity_v1.2.pdf,
!                   available from http://quake.wr.usgs.gov/~boore/daves_notes.php.
!        04/29/15 - Replaced comment characters * or C with ! (The Fortran 95 standard)
!        07/09/15 - Replace default relation with new one from daves_notes_on_relating_density_to_velocity_v2.0.pdf
!        11/08/15 - Replace default relation with new one from daves_notes_on_relating_density_to_velocity_v3.0.pdf
!        07/09/18 - The only change was that the default relation has now been published as 
!                   Boore (BSSA, 2016, pp. 316--320).  The equations are the same as for the 11/08/15 revision.


      character units_c*(*), units_uc*3
      logical specify_dens_coeff, firstcall
      data firstcall/.true./

!      if (firstcall) then
!        firstcall = .false.
!        if (.not. specify_dens_coeff) then
!          dens_low = 2.5
!          vel_low = 0.3
!          dens_high = 2.8
!          vel_high = 3.5
!        end if
!        dd_dv = (dens_high - dens_low)/(vel_high - vel_low)
!      end if

      call trim_c(units_c, nc_units_c)
      units_uc = ' '
      units_uc = units_c(1:3)
      call upstr(units_uc)
      

      if (.not. specify_dens_coeff) then
        if (units_uc(1:3) .eq. 'KPS') then
           vel = vels
        else if (units_uc(1:3) .eq. 'MPS') then
           vel = vels/1000.0
        else
           print *,' ERROR, units_uc = '//units_uc(1:3)//
     :             ' which is not a valid string; QUITTING'
           stop
        end if
        
        if (vel .lt. 0.30) then
          density = 1.0 + (1.53*vel**0.85)/(0.35+1.889*vel**1.7)
!          density = 1.0 + (1.22*vel)/(0.0948+vel)
        else
          vp = 0.9409 + 2.0947*vel -0.8206*vel**2 + 0.2683*vel**3
     :                                            - 0.0251*vel**4
          if (vel. ge. 0.30 .and. vel. lt. 3.55) then
            density = 1.74 * vp**0.25
          else
            density = 1.6612*vp - 0.4721*vp**2 + 0.0671*vp**3
     :                          - 0.0043*vp**4 + 0.000106*vp**5 
          end if
        end if
        
      else
        if (vels .lt. vel_low) then
          density = dens_low
        else if (vels .gt. vel_high) then
          density = dens_high
        else
          dd_dv = (dens_high - dens_low)/(vel_high - vel_low)
          density = dens_low + (vels - vel_low)*dd_dv
        end if
      end if

      return
      end
! --------------------------- END DENSITY ----------------------------
!************************************************
      subroutine nhrpclss(avgv, class)
!**   Classify soil type based on velocity over top 30 m.
! Dates: 08/28/03 - written by D. Boore, based on subroutine class in program
!                   assgncls
!        04/29/15 - Replaced comment characters * or C with ! (The Fortran 95 standard)

      character*1 class
      if (avgv .gt. 1500.) then
        class = 'A'
      else if (avgv .le. 1500. .and. avgv .gt. 760.) then
        class = 'B'
      else if (avgv .le.  760. .and. avgv .gt. 360.) then
        class = 'C'
      else if (avgv .le.  360. .and. avgv .ge. 180.) then
        class = 'D'  ! .ge. 180 according to NEHRP document
      else                !  (avgv .lt.  180.) is implied
        class = 'E'
      end if

      return
      end 
!************************************************
!* --------------------- BEGIN LOCATE -----------------
      SUBROUTINE locate(xx,n,x,j)
      
! Comments added by D. Boore on 26feb2010:
!  finds j such that xx(j) < x <= xx(j+1)
!  EXCEPT if x = xx(1), then j = 1 (logically it would be 0 from
!  the above relation, but this is the same returned value of j
!  for a point out of range).
!  Also, if x = xx(n), j = n-1, which is OK
!  Note that j = 0 or j = n indicates that x is out of range.
!
! See the program test_locate.for to test this routine.

! Dates: 04/28/15 - Replaced comment characters * or C with ! (The Fortran 95 standard)

      INTEGER j,n
      REAL x,xx(n)
      INTEGER jl,jm,ju
      jl=0
      ju=n+1
10    if(ju-jl.gt.1)then
        jm=(ju+jl)/2
        if((xx(n).ge.xx(1)).eqv.(x.ge.xx(jm)))then
          jl=jm
        else
          ju=jm
        endif
      goto 10
      endif
      if(x.eq.xx(1))then
        j=1
      else if(x.eq.xx(n))then
        j=n-1
      else
        j=jl
      endif
      return
      END
!* --------------------- END LOCATE -----------------

! ------------------  BEGIN TT_FOR_PIECEWISE_CONTINUOUS_MODEL ---------------------------------     
!
      function TT_FOR_PIECEWISE_CONTINUOUS_MODEL(depth4tt, depth_model, 
     :         vel_model, tt_model, ndepths_model, v_halfspace)
     
! Dates: 02/16/16 - Written by D.M. Boore


      implicit none

      real tt_for_piecewise_continuous_model     
      real depth4tt, depth_model(*), vel_model(*),slope_v, tt_model(*),
     : delta_depth, delta_tt, vel_depth, v_halfspace
      
      integer ndepths_model, j
      
! Find the index j of the model point with a depth above the desired depth:
      call locate(depth_model,ndepths_model,depth4tt,j)

! Compute the travel time corresponding to depth:
      delta_depth = depth4tt - depth_model(j)
      IF (j == ndepths_model) THEN            ! into the halfspace
        delta_tt = delta_depth/v_halfspace
      ELSE 
        IF(delta_depth == 0.0) THEN  ! no change to model tt
          delta_tt = 0.0
        ELSE IF (vel_model(j) == vel_model(j+1)) THEN        ! constant velocity
          delta_tt = delta_depth/vel_model(j)
        ELSE
          slope_v = (vel_model(j+1)   - vel_model(j))/
     :                  (depth_model(j+1) - depth_model(j))
          vel_depth = vel_model(j)+slope_v*delta_depth
          delta_tt =  
     :              (1.0/slope_v)*alog(vel_depth/vel_model(j))
        END IF
      END IF

      tt_for_piecewise_continuous_model = tt_model(j) + delta_tt
      
      return
      
      end
!
! ------------------  END TT_FOR_PIECEWISE_CONTINUOUS_MODEL ---------------------------------     

      
      ! begin subroutine interpolate_1d
      subroutine interpolate_1d(x, y, npts, ynull,
     :                          x_interp, y_interp)
      
! Returns y_interp = ynull if x_interp is outside of the range of x.
! The program assumes that x is ordered from small to large.
      
! Comments for locate:
!  finds j such that xx(j) < x <= xx(j+1)
!  EXCEPT if x = xx(1), then j = 1 (logically it would be 0 from
!  the above relation, but this is the same returned value of j
!  for a point out of range).
!  Also, if x = xx(n), j = n-1, which is OK
!  Note that j = 0 or j = n indicates that x is out of range.
!
! See the program test_locate.for to test the routine LOCATE.

! Dates: 05/27/10 - Written by D. Boore,
!                   using the equations in section 3.6 of 
!                   Press et al, Numerical Recipes (1992).

      real x(npts), y(npts), ynull,
     :     x_interp, y_interp, t, y1, y2
      
      integer j
      
      call locate(x,npts,x_interp,j)

      if (j == 0 .or. j == npts) then ! out of range
        y_interp = ynull
        return
      end if
      
      t = (x_interp - x(j))/(x(j+1) - x(j))
      
      y1 = y(j)
      y2 = y(j+1)
      
      y_interp = (1.0-t)*y1 + t*y2
      
      return
      end
! end subroutine interpolate_1d
  
